import os.path
import pathlib
import json
import mysql.connector as mysql
from mysql.connector import pooling
import re
import codecs
import sys
import traceback
import numpy as np
import subprocess
from datetime import datetime
from contextlib import contextmanager
import warnings
from rappmysql.mysqlquerys import connect
import rappmysql

compName = os.getenv('COMPUTERNAME')


import os
import re
import sys
import traceback
import subprocess
from datetime import datetime
from mysql.connector import pooling
import mysql.connector as mysql


class DataBase:
    def __init__(self, credentials):
        self.db_name = credentials['database']
        self.pool = pooling.MySQLConnectionPool(
            pool_name="mypool",
            pool_size=3,
            pool_reset_session=True,
            **credentials
        )

    def get_connection(self):
        """Get a connection from the pool"""
        return self.pool.get_connection()

    @property
    def is_connected(self):
        db = self.get_connection()
        try:
            return db.is_connected()
        finally:
            db.close()

    @property
    def dataBaseVersion(self):
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            cur.execute('SELECT version()')
            return cur.fetchone()
        finally:
            cur.close()
            conn.close()

    @property
    def allAvailableTablesInDatabase(self):
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            cur.execute("SHOW TABLES")
            tables = [row[0] for row in cur.fetchall()]
            return sorted(tables)
        finally:
            cur.close()
            conn.close()

    @property
    def allAvailableDatabases(self):
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            cur.execute("SHOW DATABASES")
            databases = [row[0] for row in cur.fetchall()]
            return sorted(databases)
        finally:
            cur.close()
            conn.close()

    def checkProcess(self):
        query = "SHOW PROCESSLIST"
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            cur.execute(query)
            records = cur.fetchall()
        finally:
            cur.close()
            conn.close()
        return records

    def killProcess(self, processes):
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            for process in processes:
                query = f"KILL {process}"
                cur.execute(query)
            conn.commit()
        finally:
            cur.close()
            conn.close()

    def killAllProcess(self):
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            query = "SHOW PROCESSLIST"
            cur.execute(query)
            records = cur.fetchall()
            for process in records:
                if process[-1] == 'SHOW PROCESSLIST':
                    continue
                kill_query = f"KILL {process[0]}"
                cur.execute(kill_query)
            conn.commit()
        finally:
            cur.close()
            conn.close()

    def run_sql_file(self, file):
        error = None
        command = ''
        try:
            with open(file, errors="ignore", encoding="utf-8") as f:
                for line in f.read().splitlines():
                    command += line
                    if ';' in line:
                        conn = self.get_connection()
                        cur = conn.cursor()
                        try:
                            cur.execute(command)
                            conn.commit()
                            command = ''
                        except mysql.Error as err:
                            error = (err.msg, command)
                            break
                        finally:
                            cur.close()
                            conn.close()
        except Exception:
            error = traceback.format_exc()
        return error

    def createTableFromFile(self, file, newTableName):
        error = None
        command = ''
        try:
            with open(file) as f:
                for line in f.read().splitlines():
                    command += line
                    if ';' in line:
                        if re.search("^CREATE TABLE", command) or \
                                re.search("^ALTER TABLE", command) or \
                                re.search("^INSERT INTO", command):
                            match = re.findall(r"\`(.+?)\`", command)
                            if match:
                                tableName = match[0]
                                command = command.replace(tableName, newTableName)
                        conn = self.get_connection()
                        cur = conn.cursor()
                        try:
                            cur.execute(command)
                            conn.commit()
                            command = ''
                        except mysql.Error as err:
                            error = (err.msg, command)
                            break
                        finally:
                            cur.close()
                            conn.close()
        except Exception:
            error = traceback.format_exc()
        return error

    def createTableList(self, fileList):
        errList = []
        # copy fileList to avoid mutation during iteration
        files_to_process = fileList[:]

        while files_to_process:
            for i, file in enumerate(files_to_process):
                error = self.createTableFromFile(file, newTableName="")  # You may need to adjust this param
                if error:
                    if error[0] == 'Cannot add foreign key constraint' or \
                            re.search("^relation.*does not exist$", error[0]):
                        errList.append(error[1])
                        files_to_process.pop(i)
                        break
                    elif 'already exists' in error[0]:
                        files_to_process.pop(i)
                        break
                    else:
                        files_to_process.pop(i)
                        break
                else:
                    files_to_process.pop(i)
                    break

        probList = []
        for com in errList:
            conn = self.get_connection()
            cur = conn.cursor()
            try:
                cur.execute(com)
                conn.commit()
            except Exception:
                error = (traceback.format_exc(), com)
                probList.append(error)
            finally:
                cur.close()
                conn.close()

        if probList:
            for i in errList:
                print(i)
        else:
            print('Successfully')

    def drop_table(self, tableName):
        query = "DROP TABLE IF EXISTS {};".format(tableName)
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            cur.execute(query)
            conn.commit()
        finally:
            cur.close()
            conn.close()

    def drop_table_list(self, tableList):
        results = []
        while tableList:
            for i, tab in enumerate(tableList):
                if tab not in self.allAvailableTablesInDatabase:
                    tableList.pop(i)
                    break
                conn = self.get_connection()
                cur = conn.cursor()
                try:
                    query = "SELECT table_name FROM information_schema.KEY_COLUMN_USAGE " \
                            "WHERE table_schema = %s AND referenced_table_name = %s "
                    cur.execute(query, (self.db_name, tab))
                    children = [row for row in cur.fetchall() if row[0]]

                    if not children:
                        cur.close()
                        conn.close()
                        self.drop_table(tab)
                        results.append(True)
                        tableList.pop(i)
                        break
                    else:
                        for child in children:
                            if child[0] not in tableList:
                                cur.close()
                                conn.close()
                                self.drop_table(tab)
                                results.append(True)
                                tableList.pop(i)
                                break
                        else:
                            # If none of the children triggered a drop, just close here and continue
                            cur.close()
                            conn.close()
                except:
                    cur.close()
                    conn.close()
                    raise
        return results

    def rename_table(self, tableName, newName):
        query = 'RENAME TABLE {} TO {}'.format(tableName, newName)
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            cur.execute(query)
            conn.commit()
        except mysql.connector.Error as err:
            print(err.msg)
        finally:
            cur.close()
            conn.close()

    def deleteAllDataInTable(self, tableList):
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            for tab in tableList:
                query = 'DELETE FROM {}'.format(tab)
                cur.execute(query)
            conn.commit()
        finally:
            cur.close()
            conn.close()

    def export_database(self, output_file):
        cmd = ['mysqldump', '-u', 'root', '-p', 'cheltuieli_desktop', '>', output_file]
        try:
            p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                 universal_newlines=True)
            out, err = p.communicate()
            if p.returncode != 0:
                print('returncode: ', p.returncode)
                print('Error: ', err)
            else:
                print('Done: {}'.format(cmd))
        except Exception:
            print('Failed to export database.')

    def show_create_table(self, tableName):
        query = 'SHOW CREATE TABLE {}'.format(tableName)
        conn = self.get_connection()
        cursor = conn.cursor()
        try:
            cursor.execute(query)
            records = cursor.fetchall()
            data = records[0][1]
            filename = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\aa.sql"
            with open(filename, "w") as FILE:
                FILE.writelines(data)
        finally:
            cursor.close()
            conn.close()

    def show_create_table_2(self, tableName):
        query = 'SELECT * FROM {}'.format(tableName)
        conn = self.get_connection()
        cursor = conn.cursor()
        try:
            cursor.execute(query)
            records = cursor.fetchall()
            data = ""
            for row in records:
                data += "INSERT INTO `" + str(tableName) + "` VALUES("
                data += ', '.join('"' + str(field) + '"' for field in row)
                data += ");\n"
            data += "\n\n"
            filename = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\bb.sql"
            with open(filename, "w") as FILE:
                FILE.writelines(data)
        finally:
            cursor.close()
            conn.close()

    def export_tables_to_sql(self, tables, output_sql_file):
        data = ""
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            for table in tables:
                cur.execute("SHOW CREATE TABLE `{}`;".format(table))
                data += "\n" + str(cur.fetchone()[1]) + ";\n\n"
                cur.execute("SELECT * FROM `{}`;".format(table))
                for row in cur.fetchall():
                    row_data = ', '.join('"{}"'.format(str(field)) for field in row)
                    data += f"INSERT INTO `{table}` VALUES({row_data});\n"
                data += "\n\n"
            with open(output_sql_file, "w") as FILE:
                FILE.writelines(data)
        finally:
            cur.close()
            conn.close()

    def backup_profile_with_files(self, tables, user_id, output_dir=None, export_files=True):
        if not output_dir:
            tim = datetime.strftime(datetime.now(), '%Y_%m_%d__%H_%M_%S')
            dir = os.path.dirname(__file__)
            output_dir_name = '{}_{}'.format(tim, '{:09d}'.format(user_id))
            output_dir = os.path.join(dir, 'static', 'backup_profile', '{:09d}'.format(user_id), output_dir_name)

        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        data = ""
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            for table in tables:
                cur.execute(f"SHOW CREATE TABLE {table}")
                create_sql = cur.fetchone()[1].replace("CREATE TABLE", "CREATE TABLE IF NOT EXISTS")
                data += "\n{};\n\n".format(create_sql)

                cur.execute(f"DESC {table}")
                cols = cur.fetchall()
                file_cols = [col[0] for col in cols if 'file' in col[0]]
                export_cols = [col[0] for col in cols if col[0] not in file_cols]

                export_cols_query = ', '.join(export_cols)
                query = f"SELECT {export_cols_query} FROM {table} WHERE {tables[table]} = {user_id};"
                cur.execute(query)
                for row in cur.fetchall():
                    values = ', '.join(
                        'NULL' if val is None else '"{}"'.format(str(val).replace('"', "'")) for val in row)
                    data += f"INSERT IGNORE INTO {table} ({export_cols_query}) VALUES({values});\n"
                data += "\n\n"

                if file_cols and export_files:
                    file_export_query = ', '.join(file_cols)
                    query = f"SELECT id, {file_export_query} FROM {table} WHERE {tables[table]} = {user_id};"
                    cur.execute(query)
                    for row in cur.fetchall():
                        row_id, file_blob, file_name = row
                        if not file_blob:
                            continue
                        file_path = os.path.join(output_dir, f"{user_id}+{table}+{row_id}+{file_name}")
                        with open(file_path, 'wb') as f:
                            f.write(file_blob)

            output_file = f"{datetime.strftime(datetime.now(), '%Y_%m_%d__%H_%M_%S')}_user_name.sql"
            with open(os.path.join(output_dir, output_file), "w", encoding="utf-8") as FILE:
                FILE.writelines(data)
        finally:
            cur.close()
            conn.close()

    def return_sql_text_bkp(self, tables):
        data = ""
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            for table in tables:
                cur.execute(f"SHOW CREATE TABLE {table}")
                create_sql = cur.fetchone()[1].replace("CREATE TABLE", "CREATE TABLE IF NOT EXISTS")
                data += "\n{};\n\n".format(create_sql)

                cur.execute(f"DESC {table}")
                cols = cur.fetchall()
                export_cols = [col[0] for col in cols if 'file' not in col[0]]
                export_cols_query = ', '.join(export_cols)

                where_clause = ' AND '.join(f"{k} = {v}" for k, v in tables[table].items())
                cur.execute(f"SELECT {export_cols_query} FROM {table} WHERE {where_clause};")
                for row in cur.fetchall():
                    values = ', '.join(
                        'NULL' if val is None else '"{}"'.format(str(val).replace('"', "'")) for val in row)
                    data += f"INSERT IGNORE INTO {table} ({export_cols_query}) VALUES({values});\n"
                data += "\n\n"
        finally:
            cur.close()
            conn.close()
        return data

    def return_sql_text(self, tables, export_files=False, export_all_users=False):
        data = ""
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            for table in tables:
                cur.execute(f"SHOW CREATE TABLE {table}")
                create_sql = cur.fetchone()[1].replace("CREATE TABLE", "CREATE TABLE IF NOT EXISTS")
                data += "\n{};\n\n".format(create_sql)

                cur.execute(f"DESC {table}")
                cols = cur.fetchall()
                file_cols = [col[0] for col in cols if 'file' in col[0]]
                export_cols = [col[0] for col in cols if col[0] not in file_cols]
                export_cols_query = ', '.join(export_cols)

                if export_all_users:
                    query = f"SELECT {export_cols_query} FROM {table}"
                else:
                    where_clause = ' AND '.join(f"{k} = {v}" for k, v in tables[table].items())
                    query = f"SELECT {export_cols_query} FROM {table} WHERE {where_clause};"
                cur.execute(query)
                for row in cur.fetchall():
                    values = ', '.join(
                        'NULL' if val is None else '"{}"'.format(str(val).replace('"', "'")) for val in row)
                    data += f"INSERT IGNORE INTO {table} ({export_cols_query}) VALUES({values});\n"
                data += "\n\n"

                if file_cols and export_files:
                    file_export_query = ', '.join(file_cols)
                    query = f"SELECT id, {file_export_query} FROM {table} WHERE {where_clause};"
                    cur.execute(query)
                    for row in cur.fetchall():
                        row_id, file_blob, file_name = row
                        if not file_blob:
                            continue
                        filename = os.path.join(export_files, f"{row_id}+{file_name}")
                        with open(filename, 'wb') as f:
                            f.write(file_blob)
        finally:
            cur.close()
            conn.close()
        return data

    def createAliasTempTable(self, tableName, colDict, origTab):
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            select_query = ', '.join(f'{col} {alias}' for col, alias in colDict.items())
            cur.execute(f"CREATE TEMPORARY TABLE {tableName} SELECT {select_query} FROM {origTab}")
            cur.execute(f"DESC {tableName}")
            tableHead = [i[0] for i in cur.fetchall()]
            cur.execute(f"SELECT * FROM {tableName}")
            table = cur.fetchall()
        finally:
            cur.close()
            conn.close()
        return tableHead, table

    def descAliasTempTable(self, tableName, colDict, origTab):
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            select_query = ', '.join(f'{col} {alias}' for col, alias in colDict.items())
            cur.execute(f"CREATE TEMPORARY TABLE {tableName} SELECT {select_query} FROM {origTab}")
            cur.execute(f"DESC {tableName}")
            tableHead = [i[0] for i in cur.fetchall()]
            cur.execute(f"SELECT * FROM {tableName}")
            table = cur.fetchall()
        finally:
            cur.close()
            conn.close()
        return tableHead, table

    # def close(self):
    #     """Close the database connection."""
    #     if self.cursor:
    #         self.cursor.close()
    #     if self.conn:
    #         self.conn.close()


class Table(DataBase):
    def __init__(self, credentials, tableName):
        super().__init__(credentials)
        self.tableName = tableName

    @property
    def noOfRows(self):
        query = 'SELECT COUNT(*) FROM {}'.format(self.tableName)
        conn = self.get_connection()
        cursor = conn.cursor()
        try:
            cursor.execute(query)
            noOfRows = cursor.fetchone()[0]
        finally:
            cursor.close()
            conn.close()
        return noOfRows

    def lastRowId(self):
        query = 'SELECT id FROM {} ORDER BY id DESC LIMIT 1'.format(self.tableName)
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            cur.execute(query)
            lastId = cur.fetchone()
            if lastId is None:
                return 0
            return lastId[0]
        finally:
            cur.close()
            conn.close()

    @property
    def columnsNames(self):
        query = 'DESC {}'.format(self.tableName)
        conn = self.get_connection()
        cursor = conn.cursor()
        try:
            cursor.execute(query)
            res = cursor.fetchall()
            cols = [col[0] for col in res]
        finally:
            cursor.close()
            conn.close()
        return cols

    @property
    def columnsDetProperties(self):
        query = 'DESC {}'.format(self.tableName)
        colNames = ['Field', 'Type', 'Null', 'Key', 'Default', 'Extra']
        conn = self.get_connection()
        cursor = conn.cursor()
        try:
            cursor.execute(query)
            res = cursor.fetchall()
            cols = {}
            for col in res:
                colName, colType, null, key, default, extra = col
                if isinstance(colType, bytes):
                    colType = colType.decode("utf-8")
                cols[colName] = [colType, null, key, default, extra]
        finally:
            cursor.close()
            conn.close()
        return cols

    @property
    def columnsProperties(self):
        query = (
            "SELECT table_name, column_name, data_type FROM INFORMATION_SCHEMA.COLUMNS WHERE table_name = '{}'").format(
            self.tableName)
        conn = self.get_connection()
        cursor = conn.cursor()
        try:
            cursor.execute(query)
            res = cursor.fetchall()
            cols = {col_name: data_type for _, col_name, data_type in res}
        finally:
            cursor.close()
            conn.close()
        return cols

    @property
    def children_tables(self):
        query = "SELECT table_name FROM information_schema.KEY_COLUMN_USAGE WHERE table_schema = %s AND referenced_table_name = %s"
        conn = self.get_connection()
        cur = conn.cursor()
        try:
            cur.execute(query, (self.db_name, self.tableName))
            children = [row[0] for row in cur.fetchall() if row[0]]
        finally:
            cur.close()
            conn.close()
        return children

    def rename_column(self, tableName, old_col_name, new_col_name):
        query = 'ALTER TABLE {} RENAME COLUMN {} TO {}'.format(tableName, old_col_name, new_col_name)
        db = self.get_connection()
        cur = db.cursor()
        try:
            cur.execute(query)
        except mysql.connector.Error as err:
            print(err.msg)
        finally:
            cur.close()
            db.close()

    def deleteRow(self, condition):
        colName, value = condition
        query = 'DELETE FROM {} WHERE {} = %s'.format(self.tableName, colName)
        db = self.get_connection()
        cursor = db.cursor()
        try:
            cursor.execute(query, (value,))
            db.commit()
        finally:
            cursor.close()
            db.close()

    def delete_multiple_rows(self, condition):
        colName, value = condition
        query = 'DELETE FROM {} WHERE {} = %s'.format(self.tableName, colName)
        db = self.get_connection()
        cursor = db.cursor()
        try:
            cursor.execute(query, (value,))
            db.commit()
        finally:
            cursor.close()
            db.close()

    def delete_multiple_rows_multiple_conditions(self, condition):
        colName, value = condition
        query = 'DELETE FROM {} WHERE {} = %s'.format(self.tableName, colName)
        db = self.get_connection()
        cursor = db.cursor()
        try:
            cursor.execute(query, (value,))
            db.commit()
        finally:
            cursor.close()
            db.close()

    def convertToBinaryData(self, filename):
        with open(filename, 'rb') as file:
            binaryData = file.read()
        return binaryData

    def addNewRow(self, columns, values):
        strCols = ', '.join(columns)
        strVals = ', '.join(['%s'] * len(columns))
        query = "INSERT INTO {} ({}) VALUES ({})".format(self.tableName, strCols, strVals)

        if isinstance(values, (int, str)):
            values = (values,)
        elif isinstance(values, tuple):
            new_vals = []
            for v in values:
                if isinstance(v, str) and os.path.isfile(v):
                    v = self.convertToBinaryData(v)
                new_vals.append(v)
            values = tuple(new_vals)

        db = self.get_connection()
        cursor = db.cursor()
        try:
            cursor.execute(query, values)
            db.commit()
        except Exception as e:
            print('Insert error:', e)
            print('Query:', query)
            print('Values:', values)
            raise
        finally:
            cursor.close()
            db.close()
        return cursor.lastrowid

    def addNewRowWithFile(self, columns, values):
        strCols = ', '.join(columns)
        strVals = ', '.join(['%s'] * len(columns))
        query = "INSERT INTO {} ({}) VALUES ({})".format(self.tableName, strCols, strVals)
        print('Executing:', query)

        if isinstance(values, (int, str)):
            values = (values,)
        elif isinstance(values, tuple):
            new_vals = []
            for v in values:
                if isinstance(v, pathlib.Path):
                    if v.is_file():
                        v = self.convertToBinaryData(str(v))
                new_vals.append(v)
            values = tuple(new_vals)

        db = self.get_connection()
        cursor = db.cursor()
        try:
            cursor.execute(query, values)
            db.commit()
            print('Executed successfully')
        except mysql.connector.Error as err:
            if err.errno == 2013 and 'Lost connection' in err.msg:
                print('Handling lost connection error...')
                cursor.execute('SET max_allowed_packet=67108864')
                db.commit()
                cursor.execute(query, values)
                db.commit()
                print('Re-executed successfully')
            else:
                print('MySQL Error:', err.errno, err.msg)
                raise
        finally:
            cursor.close()
            db.close()
        return cursor.lastrowid

    def insertColumns(self, column_name, column_definition, afterCol):
        if afterCol == 'FIRST':
            query = 'ALTER TABLE {} ADD COLUMN {} {} FIRST'.format(self.tableName, column_name, column_definition)
        else:
            query = 'ALTER TABLE {} ADD COLUMN {} {} AFTER {}'.format(self.tableName, column_name, column_definition,
                                                                      afterCol)
        db = self.get_connection()
        cursor = db.cursor()
        try:
            cursor.execute(query)
            db.commit()
        finally:
            cursor.close()
            db.close()

    def returnAllRecordsFromTable(self):
        db = self.get_connection()
        cur = db.cursor()
        try:
            query = 'SELECT * FROM {}'.format(self.tableName)
            cur.execute(query)
            records = cur.fetchall()
        finally:
            cur.close()
            db.close()
        return records

    def returnAllRecordsFromTableExceptBlob(self):
        db = self.get_connection()
        cur = db.cursor()
        try:
            query = 'SELECT '
            for col_name, prop in self.columnsDetProperties.items():
                if prop[0] == 'longblob':
                    continue
                query += f'{col_name}, '
            query = query.rstrip(', ')
            query += ' FROM {}'.format(self.tableName)
            print('Executing:', query)
            cur.execute(query)
            records = cur.fetchall()
        finally:
            cur.close()
            db.close()
        return records

    def returnLastRecords(self, column, noOfRows2Return):
        db = self.get_connection()
        cur = db.cursor()
        try:
            query = 'SELECT * FROM {} ORDER BY {} DESC LIMIT %s'.format(self.tableName, column)
            cur.execute(query, (noOfRows2Return,))
            rows = cur.fetchall()
        finally:
            cur.close()
            db.close()
        return rows

    def filterRows(self, matches, order_by=None):
        filterText = ''
        for match in matches:
            search_col, search_key = match
            if isinstance(search_key, tuple):
                min, max = search_key
                new = "{} >= '{}' AND {} <= '{}' AND ".format(search_col, min, search_col, max)
                filterText += new
            elif isinstance(search_key, list):
                new = "{} in {} AND ".format(search_col, tuple(search_key))
                filterText += new
            elif search_key == 'None' or search_key is None:
                new = "{} IS NULL AND ".format(search_col)
                filterText += new
            else:
                new = "{} = '{}' AND ".format(search_col, search_key)
                filterText += new

        query = 'SELECT '
        for col_name, prop in self.columnsDetProperties.items():
            if prop[0] == 'longblob':
                continue
            query += '{}, '.format(col_name)
        query = query[:-2]
        query += ' FROM {} WHERE {} '.format(self.tableName, filterText[:-4])

        if order_by:
            col, order = order_by
            query += 'ORDER BY {} {}'.format(col, order)

        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def returnRowsWhere(self, matches):
        has_longblob_col = False
        cols_without_longblob = ''
        for col_name, prop in self.columnsDetProperties.items():
            if prop[0] == 'longblob':
                has_longblob_col = True
            else:
                cols_without_longblob += '{}, '.format(col_name)
        cols_without_longblob = cols_without_longblob[:-2]

        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, (str, int)):
                query = "SELECT * FROM {} WHERE {} = '{}'".format(self.tableName, searchCol, searchKey)
            elif isinstance(searchKey, tuple):
                query = "SELECT * FROM {} WHERE {} IN '{}'".format(self.tableName, searchCol, searchKey)
            elif searchKey is None:
                query = "SELECT * FROM {} WHERE {} IS NULL".format(self.tableName, searchCol)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                if searchKey is None:
                    text += '{} IS NULL AND '.format(searchCol)
                else:
                    text += '{} = "{}" AND '.format(searchCol, searchKey)
            query = "SELECT * FROM {} WHERE ".format(self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        if has_longblob_col:
            query = query.replace('*', cols_without_longblob)

        db = self.get_connection()
        cursor = db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        db.close()
        return list(records)

    def returnRowsLike(self, column, keyWord):
        query = "SELECT * FROM {} WHERE {} LIKE '%{}%'".format(self.tableName, column, keyWord)
        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def returnRowsYoungerThan(self, column, timeP):
        query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, column, column)
        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query, (timeP,))
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def returnRowsInInterval(self, startColumn, startTime, endColumn, endTime):
        query = "SELECT * FROM {} WHERE {} > %s OR {} < %s".format(self.tableName, startColumn, endColumn)
        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query, (startTime, endTime))
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def returnRowsWhereValueIsInIntervalAND(self, colWhere, keyWord, startColumn, valueInInterval, endColumn):
        query = "SELECT * FROM {} WHERE {} = %s AND %s > {} AND {} > %s".format(self.tableName, colWhere, startColumn,
                                                                                endColumn)
        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query, (keyWord, valueInInterval, valueInInterval))
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def returnCellWhereValueIsInIntervalAND(self, col2return, colWhere, keyWord, startColumn, valueInInterval,
                                            endColumn):
        query = ("SELECT {} FROM {} WHERE {} = %s AND %s > {} AND ({} > %s OR {} IS NULL)"
                 .format(col2return, self.tableName, colWhere, startColumn, endColumn, endColumn))
        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query, (keyWord, valueInInterval, valueInInterval))
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def returnRowsQuery(self, matches, order_by=None):
        filterText = ''
        for match in matches:
            search_col, sign, search_value = match
            if sign == 'LIKE':
                filterText += "{} {} '%{}%' AND ".format(search_col, sign, search_value)
            elif sign == '==' and search_value is None:
                filterText += "{} IS NULL AND ".format(search_col)
            elif sign == '!=' and search_value is None:
                filterText += "{} IS NOT NULL AND ".format(search_col)
            else:
                filterText += "{} {} '{}' AND ".format(search_col, sign, search_value)

        query = 'SELECT '
        for col_name, prop in self.columnsDetProperties.items():
            if prop[0] == 'longblob':
                continue
            query += '{}, '.format(col_name)
        query = query[:-2]
        query += ' FROM {} WHERE {} '.format(self.tableName, filterText[:-4])
        if order_by:
            col, order = order_by
            query += 'ORDER BY {} {}'.format(col, order)

        # print('__query__', query)
        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def return_from_table(self, matches, colname=None, with_files=False, order_by=None):
        filterText = ''
        for match in matches:
            search_col, sign, search_value = match
            if sign == 'LIKE':
                filterText += "{} {} '%{}%' AND ".format(search_col, sign, search_value)
            elif sign == '==' and search_value is None:
                filterText += "{} IS NULL AND ".format(search_col)
            elif sign == '!=' and search_value is None:
                filterText += "{} IS NOT NULL AND ".format(search_col)
            else:
                filterText += "{} {} '{}' AND ".format(search_col, sign, search_value)

        if colname:
            query = 'SELECT {} '.format(colname)
        else:
            if with_files:
                query = 'SELECT * '
            else:
                query = 'SELECT '
                for col_name, prop in self.columnsDetProperties.items():
                    if prop[0] == 'longblob':
                        continue
                    query += '{}, '.format(col_name)
                query = query[:-2]

        query += ' FROM {} WHERE {} '.format(self.tableName, filterText[:-4])
        if order_by:
            col, order = order_by
            query += 'ORDER BY {} {}'.format(col, order)

        print('__query__', query)
        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def returnRowsOfYear(self, startColumn, startTime, endColumn, endTime):
        query = "SELECT * FROM {} WHERE {} >= %s AND {} <= %s".format(self.tableName, startColumn, endColumn)
        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query, (startTime, endTime))
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def returnRowsOutsideInterval(self, startColumn, startTime, endColumn, endTime):
        query = "SELECT * FROM {} WHERE {} < %s AND {} > %s".format(self.tableName, startColumn, endColumn)
        db = self.get_connection()
        cur = db.cursor()
        cur.execute(query, (startTime, endTime))
        records = cur.fetchall()
        cur.close()
        db.close()
        return records

    def get_column_type(self, column):
        colProps = self.columnsProperties[column]
        colType = colProps[0]
        return colType

    def modify2AutoIncrement(self, column, colType):
        query = 'ALTER TABLE {} MODIFY {} {} AUTO_INCREMENT;'.format(self.tableName, column, colType)
        print(query)
        db = self.get_connection()
        cursor = db.cursor()
        cursor.execute(query)
        db.commit()
        cursor.close()
        db.close()

    def modifyType(self, column, colType):
        query = 'ALTER TABLE {} MODIFY {} {};'.format(self.tableName, column, colType)
        db = self.get_connection()
        cursor = db.cursor()
        cursor.execute(query)
        db.commit()
        cursor.close()
        db.close()

    def changeCellContent(self, column2Modify, val2Moify, refColumn, refValue):
        db = self.get_connection()
        cursor = db.cursor()
        try:
            if isinstance(refValue, tuple):
                query = "UPDATE {} SET {} = {} WHERE {} IN {}".format(self.tableName, column2Modify, val2Moify,
                                                                      refColumn, refValue)
                cursor.execute(query)
                db.commit()
                cursor.close()
                db.close()
                return
            else:
                query = "UPDATE {} SET {} = %s WHERE {} = %s".format(self.tableName, column2Modify, refColumn)
            print(query)
            print('val2Moify', val2Moify)

            if isinstance(val2Moify, str):
                if os.path.isfile(val2Moify):
                    print('aaaaa', val2Moify, type(val2Moify))
                    val2Moify = self.convertToBinaryData(val2Moify)
            vals = (val2Moify, int(refValue))
            cursor.execute(query, vals)
            db.commit()
            cursor.close()
            db.close()
        except mysql.Error as err:
            print('++', err)
            print('query', query)
            print('vals', vals)
            print('ERROR mysql.Error: ', err.msg)
            cursor.close()
            db.close()

    def dropColumn(self, column2Del):
        query = "ALTER TABLE {} DROP COLUMN {};".format(self.tableName, column2Del)
        print(query)
        db = self.get_connection()
        cursor = db.cursor()
        cursor.execute(query)
        db.commit()
        cursor.close()
        db.close()

    def executeQuery(self, query):
        print(sys._getframe().f_code.co_name)
        db = self.get_connection()
        cursor = db.cursor()
        if isinstance(query, str):
            commands = query.split(';')
        for command in commands:
            print('executing command: ', command)
            cursor.execute(command)
        cursor.close()
        db.close()

    def convertDatumFormat4SQL(self, datum):
        for fmt in ('%Y-%m-%d', '%d.%m.%Y', '%d/%m/%Y', '%m/%d/%Y', '%d.%m.%y'):
            try:
                newDate = datetime.strptime(datum, fmt)
                return newDate.date()
            except ValueError:
                pass
        raise ValueError('no valid date format found: {}'.format(datum))

    def convertDateTimeFormat4SQL(self, datum):
        for fmt in ('%Y/%m/%d %H:%M', '%Y-%m-%d %H:%M'):
            try:
                newDate = datetime.strptime(datum, fmt)
                return newDate
            except ValueError:
                pass
        raise ValueError('no valid date format found: {}'.format(datum))

    def convertTimeFormat4SQL(self, time):
        for fmt in ('%H:%M', '%H:%M:%S'):
            try:
                newDate = datetime.strptime(time, fmt)
                return newDate.time()
            except ValueError:
                pass
        raise ValueError('no valid date format found')

    def returnColumn(self, col):
        query = 'SELECT {} FROM {}'.format(col, self.tableName)
        db = self.get_connection()
        cursor = db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        db.close()
        values = []
        for i in records:
            values.append(i[0])
        return values

    def returnColumns(self, cols):
        strTableHead = ''
        for col in cols:
            strTableHead += '{}, '.format(col)
        strTableHead = strTableHead[:-2]

        query = 'SELECT {} FROM {}'.format(strTableHead, self.tableName)
        db = self.get_connection()
        cursor = db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        db.close()
        values = []
        for i in records:
            values.append(i)
        return values

    def returnCellsWhere(self, col, matches):
        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT {} FROM {} WHERE {} = '{}'".format(col, self.tableName, searchCol, searchKey)
            if isinstance(searchKey, tuple):
                query = "SELECT {} FROM {} WHERE {} IN {}".format(col, self.tableName, searchCol, searchKey)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                new = '{} = "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT {} FROM {} WHERE ".format(col, self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        db = self.get_connection()
        cursor = db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        db.close()
        values = []
        for i in records:
            values.append(i[0])
        return values

    def returnCellsWhereDiffrent(self, col, matches):
        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT {} FROM {} WHERE {} != '{}'".format(col, self.tableName, searchCol, searchKey)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                new = '{} != "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT {} FROM {} WHERE ".format(col, self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        db = self.get_connection()
        cursor = db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        db.close()
        values = []
        colType = self.get_column_type(col)
        for i in records:
            if colType == 'json':
                values.append(json.loads(i[0]))
            else:
                values.append(i[0])
        return values

    def returnColsWhere(self, cols, matches):
        relCols = ''
        for col in cols:
            relCols += '{}, '.format(col)
        relCols = relCols[:-2]

        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT {} FROM {} WHERE {} = '{}'".format(relCols, self.tableName, searchCol, searchKey)
            if isinstance(searchKey, tuple):
                query = "SELECT {} FROM {} WHERE {} IN '{}'".format(relCols, self.tableName, searchCol, searchKey)
            if searchKey is None:
                query = "SELECT {} FROM {} WHERE {} IS NULL".format(relCols, self.tableName, searchCol)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                if searchKey is None:
                    new = '{} IS NULL AND '.format(searchCol)
                else:
                    new = '{} = "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT {} FROM {} WHERE ".format(relCols, self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        db = self.get_connection()
        cursor = db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        db.close()
        values = []
        for i in records:
            values.append(i)
        return values

    def write_file(self, data, filename):
        with open(filename, 'wb') as file:
            file.write(data)

    def compare_sql_file_to_sql_table(self, sql_file):
        f = open(sql_file)
        same = True
        diffrences = {}
        for line in f.read().splitlines():
            spl_row = line.split()
            if spl_row and '`' in spl_row[0]:
                sql_file_col = spl_row[0].strip('`')
                if sql_file_col in self.columnsDetProperties.keys():
                    col_type_file = spl_row[1]
                    col_type_sql = self.columnsDetProperties[sql_file_col][0]
                    if col_type_file != col_type_sql:
                        txt = 'Column {} has a different format in file than in sql table'.format(sql_file_col)
                        same = False
                        diffrences[sql_file_col] = txt
                else:
                    print('sql_file_col', sql_file_col)
                    print('self.columnsDetProperties.keys()', self.columnsDetProperties.keys())
                    raise RuntimeError('Column {} missing in sql table file {}'.format(sql_file_col, sql_file))
        f.close()
        if not same:
            return diffrences


def convertDatumFormat4SQL(datum):
    # print(sys._getframe().f_code.co_name)
    # newDate = datetime.strptime(datum, '%d.%m.%y')
    for fmt in ('%Y%m%d','%Y-%m-%d', '%d.%m.%Y', '%d/%m/%Y', '%m/%d/%Y', '%d.%m.%y'):
        try:
            newDate = datetime.strptime(datum, fmt)
            return newDate.date()
        except ValueError:
            pass
    raise ValueError('no valid date format found: {}'.format(datum))


def convertDateTimeFormat4SQL(datum):
    # print(sys._getframe().f_code.co_name)
    # newDate = datetime.strptime(datum, '%d.%m.%y') 2024/09/21 11:10
    for fmt in ('%Y/%m/%d %H:%M', '%Y-%m-%d %H:%M', '%Y%m%d_%H%M%S'):
        try:
            newDate = datetime.strptime(datum, fmt)
            return newDate
        except ValueError:
            pass
    raise ValueError('no valid date format found: {}'.format(datum))


if __name__ == '__main__':


    # if compName == 'DESKTOP-5HHINGF':
    #     ini_file = r"D:\Python\MySQL\cheltuieli.ini"
    # else:
    #     ini_file = r"C:\_Development\Diverse\pypi\cfgm.ini"
    #     # ini_file = r"C:\_Development\Diverse\pypi\cfgmRN.ini"

    ini_file = rappmysql.ini_chelt

    conf = connect.Config(ini_file)
    # print(conf.credentials)

    db = DataBase(conf.credentials)
    print(db.is_connected)

    # tables_dict = {'users': 'id',
    #                'all_cars': 'user_id',
    #                'masina': 'id_users'}
    # tables_dict = {'users': 'id',
    #                'banca': 'id_users',
    #                'income': 'id_users',
    #                'real_expenses_1': 'id_users',
    #                'chelt_plan': 'id_users',
    #                }

    # tables_dict = {'users': 'id',
    #                'banca': 'id_users',
    #                'chelt_plan': 'id_users',
    #                'knowntrans': 'id_users',
    #                'income': 'id_users',
    #                'deubnk': 'id_users',
    #                'n26': 'id_users',
    #                'sskm': 'id_users',
    #                'plan_vs_real': 'id_users',
    #                'imported_csv': 'id_users'
    #                }

    # output_dir = r'C:\_Development\Diverse\pypi\radu\masina\src\masina\static\backup_profile\000000001\000000001'
    # db.backup_profile_with_files(tables_dict, user_id=1, output_dir=output_dir, export_files=True)

    # db.drop_table_list(tab_list)

    matches = [('type', 'benzina'),
               ('id_users', 1),
               ('id_all_cars', 1)]

    ini_file = rappmysql.ini_masina

    conf = connect.Config(ini_file)
    credentials = conf.credentials
    alimentari = Table(credentials, 'masina')
    rows = alimentari.returnRowsWhere(matches)
    for ii in rows:
        print(ii)


import csv
import os.path
import traceback
import numpy as np
import datetime as dt
from datetime import datetime, timedelta
import time
import sys
import json
import shutil
from flask_login import UserMixin
from werkzeug.security import generate_password_hash, check_password_hash
import zipfile
import pathlib
import rappmysql
from rappmysql.mysqlquerys import connect, mysql_rm
# from rappmysql.aeroclub.aeroclub import AeroclubApp
# from rappmysql.masina.auto import AutoApp
# from rappmysql.cheltuieli.chelt_plan import CheltApp
import rappmysql.mruser as mruser

compName = os.getenv('COMPUTERNAME')

np.set_printoptions(linewidth=250)

app_tables_users = {'users': 'id', 'user_apps': 'id_users'}
app_users_gui = ['login_window.ui']


def calculate_last_day_of_month(mnth, year):
    if mnth < 12:
        # lastDayOfMonth = datetime(datetime.now().year, mnth + 1, 1) - timedelta(days=1)
        lastDayOfMonth = datetime(year, mnth + 1, 1) - timedelta(days=1)
        lastDayOfMonth = lastDayOfMonth.day
    elif mnth == 12:
        lastDayOfMonth = 31
    return lastDayOfMonth


def calculate_today_plus_x_days(x_days):
    result = datetime(datetime.now().year, datetime.now().month, datetime.now().day) + timedelta(days=int(x_days))
    return result


def default_interval():
    # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    print('Caller : ', sys._getframe().f_back.f_code.co_name)
    startDate = datetime(datetime.now().year, datetime.now().month, datetime.now().day)
    if datetime.now().month != 12:
        mnth = datetime.now().month + 1
        lastDayOfMonth = datetime(datetime.now().year, mnth, 1) - timedelta(days=1)
    else:
        lastDayOfMonth = datetime(datetime.now().year + 1, 1, 1) - timedelta(days=1)

    return startDate, lastDayOfMonth


def get_monthly_interval(month: str, year):
    # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    mnth = datetime.strptime(month, "%B").month
    startDate = datetime(year, mnth, 1)

    if mnth != 12:
        lastDayOfMonth = datetime(year, mnth + 1, 1) - timedelta(days=1)
    else:
        lastDayOfMonth = datetime(year + 1, 1, 1) - timedelta(days=1)

    return startDate.date(), lastDayOfMonth.date()


def convert_to_display_table(tableHead, table, displayTableHead):
    newTableData = np.empty([table.shape[0], len(displayTableHead)], dtype=object)
    for i, col in enumerate(displayTableHead):
        indxCol = tableHead.index(col)
        newTableData[:, i] = table[:, indxCol]
    return displayTableHead, newTableData


class DB_Connection:
    def __init__(self, ini_users):
        if isinstance(ini_users, dict):
            self.credentials = ini_users
        else:
            self.conf = connect.Config(ini_users)
            self.credentials = self.conf.credentials
        self.users_db = mysql_rm.DataBase(self.credentials)
        self.users_table = mysql_rm.Table(self.credentials, 'users')
        self.user_apps_table = mysql_rm.Table(self.credentials, 'user_apps')

    def close_connection(self):
        self.users_db.db.close()


class CheckUsersRequiredFiles:
    def __init__(self, ini_file):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        if isinstance(ini_file, dict):
            self.credentials = ini_file
        else:
            self.conf = connect.Config(ini_file)
            self.credentials = self.conf.credentials

        self.users_db = None
        path2GUI = pathlib.Path(__file__)
        self.path2GUI = path2GUI.resolve(path2GUI).parent / 'GUI'
        self.pth2SQLtables = os.path.join(os.path.dirname(__file__), 'static', 'sql')

        # self.check_connec_to_users_db()
        # self.check_users_sql_gui_files()
        # self.check_tabels_in_users_db()
        # self.connect_to_all_databases()

    def check_connec_to_users_db(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        if not isinstance(self.credentials, dict):
            raise RuntimeError('Credentials not dict')
        self.users_db = mysql_rm.DataBase(self.credentials)
        if not self.users_db.is_connected:
            raise RuntimeError('Could not connect to database')
        print('Connected to users_db:', self.users_db.is_connected)

    def check_users_sql_gui_files(self, ):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        for table in app_tables_users.keys():
            sql_file = os.path.join(self.pth2SQLtables, '{}_template.sql'.format(table))
            # print('##sql_table_template##', sql_file)
            if not os.path.exists(sql_file):
                raise FileNotFoundError('{}'.format(sql_file))

        for gui in app_users_gui:
            sql_file = os.path.join(self.path2GUI, gui)
            # print('##gui_file##', sql_file)
            if not os.path.exists(sql_file):
                raise FileNotFoundError('{}'.format(sql_file))

    def check_tabels_in_users_db(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        for table in app_tables_users.keys():
            # print('##table##', table)
            sql_file = os.path.join(self.pth2SQLtables, '{}_template.sql'.format(table))
            if table not in self.users_db.allAvailableTablesInDatabase:
                print('Table {} not in database...creating it'.format(table))
                self.users_db.createTableFromFile(sql_file, table)
            else:
                # exec("sqltable = mysql_rm.Table(self.credentials, '{}')".format(table))
                varName = 'table_{}'.format(table)
                # print("{} = mysql_rm.Table(self.credentials, '{}')".format(varName, table))
                # print('ÄÄ', varName)
                loc = locals()
                # print(loc)
                exec("{} = mysql_rm.Table(self.credentials, '{}')".format(varName, table), globals(), loc)
                varName = loc[varName]
                same = varName.compare_sql_file_to_sql_table(sql_file)
                if same is not True:
                    print(same)

    def connect_to_all_databases(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        conf = connect.Config(rappmysql.ini_masina)
        credentials_masina = conf.credentials
        self.auto_db = mysql_rm.DataBase(credentials_masina)
        if not self.auto_db.is_connected:
            raise RuntimeError('Could not connect to database')
        print('Connected to database auto_db:', self.auto_db.db_name)

        conf = connect.Config(rappmysql.ini_chelt)
        credentials_chelt = conf.credentials
        self.chelt_db = mysql_rm.DataBase(credentials_chelt)
        if not self.chelt_db.is_connected:
            raise RuntimeError('Could not connect to database')
        print('Connected to database chelt_db:', self.chelt_db.db_name)

        conf = connect.Config(rappmysql.ini_aeroclub)
        credentials_aeroclub = conf.credentials
        self.aeroclub_db = mysql_rm.DataBase(credentials_aeroclub)
        if not self.aeroclub_db.is_connected:
            raise RuntimeError('Could not connect to database')
        print('Connected to database aeroclub_db:', self.aeroclub_db.db_name)


class Users(UserMixin):
    def __init__(self, user_name, users_table, user_apps_table):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        super(UserMixin).__init__()
        # CheckUsersRequiredFiles.__init__(self, ini_users)
        self.user_name = user_name
        self.auto_app = None
        self.chelt_app = None
        self.aeroclub_app = None
        # if isinstance(ini_users, dict):
        #     self.credentials = ini_users
        # else:
        #     self.conf = connect.Config(ini_users)
        #     self.credentials = self.conf.credentials
        self.users_table = users_table
        self.user_apps_table = user_apps_table

    # def init_app(self, app_name, ini_file):
    #     print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
    #     if app_name == 'cheltuieli':
    #         self.chelt_app = CheltApp(ini_file, self.id)
    #     elif app_name == 'auto':
    #         self.auto_app = AutoApp(ini_file, self.id)
    #     elif app_name == 'aeroclub':
    #         self.aeroclub_app = AeroclubApp(ini_file, self.id)

    def add_user(self, register_details):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        cols = []
        vals = []
        for k, v in register_details.items():
            cols.append(k)
            vals.append(v)
        user_id = self.users_table.addNewRow(cols, tuple(vals))
        return user_id

    def add_application(self, app_name):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        cols = ['id_users', 'app_name']  # , 'app_credentials'
        vals = [self.id, app_name]  # , str(new_credentials)
        id_app_row = self.user_apps_table.addNewRow(cols, tuple(vals))
        return id_app_row

    def run_sql_query(self, file):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        self.users_db.run_sql_file(file)

    @property
    def all_users(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        all_users = self.users_table.returnColumn('username')
        return all_users

    @property
    def all_possible_applications(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        all_possible_applications = self.user_apps_table.returnColumn('app_name')
        all_possible_applications = list(set(all_possible_applications))
        all_possible_applications = ['cheltuieli', 'masina', 'aeroclub']
        # all_possible_applications = rappmysql.subpackages
        return all_possible_applications

    @property
    def unused_applications(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        unused_applications = set(self.applications) ^ set(self.all_possible_applications)
        # unused_applications = ['cheltuieli', 'masina']
        return list(unused_applications)

    @property
    def id(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        matches = ('username', self.user_name)
        user_id = self.users_table.returnCellsWhere('id', matches)[0]
        return user_id

    @property
    def valid_user(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        all_users = self.users_table.returnColumn('username')
        if self.user_name in all_users:
            return True
        else:
            return False

    @property
    def admin(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        if self.valid_user:
            matches = ('id', self.id)
            user_type = self.users_table.returnCellsWhere('user_type', matches)[0]
            if user_type == 'admin':
                return True
            else:
                return False

    @property
    def applications(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        matches = ('id_users', self.id)
        user_apps = self.user_apps_table.returnCellsWhere('app_name', matches)
        return list(user_apps)

    @property
    def hashed_password(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        matches = ('username', self.user_name)
        hashed_password = self.users_table.returnCellsWhere('password', matches)[0]
        return hashed_password

    def verify_password(self, password):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        return check_password_hash(self.hashed_password, password)

    def zip_profile_files(self, src_dir, output_file):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        relroot = os.path.abspath(os.path.join(src_dir, os.pardir))
        with zipfile.ZipFile(output_file, "w", zipfile.ZIP_DEFLATED) as zip:
            for root, dirs, files in os.walk(src_dir):
                # add directory (needed for empty dirs)
                zip.write(root, os.path.relpath(root, relroot))
                for file in files:
                    filename = os.path.join(root, file)
                    if os.path.isfile(filename):  # regular files only
                        arcname = os.path.join(os.path.relpath(root, relroot), file)
                        zip.write(filename, arcname)
        return output_file

    def unzip_profile_files(self, src_file, output_dir):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        with zipfile.ZipFile(src_file, 'r') as zip_ref:
            zip_ref.extractall(output_dir)
        return output_dir

    def erase_all_traces(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        self.chelt_db.drop_table_list(list(rappmysql.cheltuieli.chelt_plan.app_real_expenses.keys()))
        self.chelt_db.drop_table_list(list(rappmysql.cheltuieli.chelt_plan.app_chelt_tables.keys()))
        self.auto_db.drop_table_list(list(rappmysql.masina.auto.app_masina_tables.keys()))
        self.aeroclub_db.drop_table_list(list(rappmysql.aeroclub.aeroclub.app_aeroclub.keys()))
        self.users_db.drop_table_list(list(app_tables_users.keys()))

    def backup(self, export_files=False, complete_database=False):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        user_tables = {}
        for tab in app_tables_users:
            if tab == 'users':
                user_tables[tab] = {'id': self.id}
                continue
            user_tables[tab] = {'id_users': self.id}
        aeroclub_tables = {}
        for tab in rappmysql.aeroclub.aeroclub.app_aeroclub:
            aeroclub_tables[tab] = {'id_users': self.id}
        masina_tables = {}
        for tab in rappmysql.masina.auto.app_masina_tables:
            masina_tables[tab] = {'id_users': self.id}
        cheltuieli_tables = {}
        for tab in rappmysql.cheltuieli.chelt_plan.app_chelt_tables:
            cheltuieli_tables[tab] = {'id_users': self.id}
        for tab in rappmysql.cheltuieli.chelt_plan.app_real_expenses:
            cheltuieli_tables[tab] = {'id_users': self.id}
        tim = datetime.strftime(datetime.now(), '%Y_%m_%d__%H_%M_%S')
        if complete_database:
            output_dir = os.path.join(os.path.dirname(__file__), 'static', 'db_backup', '{}'.format(tim))
        else:
            output_dir = os.path.join(os.path.dirname(__file__), 'static', 'backup_complete_profile',
                                      '{:09d}'.format(self.id),
                                      '{}_{:09d}'.format(tim, self.id))
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        complete_sql = ''
        if export_files:
            users_sql_query = self.users_db.return_sql_text(user_tables, export_files=output_dir, export_all_users=complete_database)
            complete_sql += users_sql_query
            masina_sql_query = self.auto_db.return_sql_text(masina_tables, export_files=output_dir, export_all_users=complete_database)
            complete_sql += masina_sql_query
            chelt_sql_query = self.chelt_db.return_sql_text(cheltuieli_tables, export_files=output_dir, export_all_users=complete_database)
            complete_sql += chelt_sql_query
            aeroclub_sql_query = self.aeroclub_db.return_sql_text(aeroclub_tables, export_files=output_dir, export_all_users=complete_database)
            complete_sql += aeroclub_sql_query
        else:
            users_sql_query = self.users_db.return_sql_text(user_tables, export_all_users=complete_database)
            complete_sql += users_sql_query
            masina_sql_query = self.auto_db.return_sql_text(masina_tables, export_all_users=complete_database)
            complete_sql += masina_sql_query
            chelt_sql_query = self.chelt_db.return_sql_text(cheltuieli_tables, export_all_users=complete_database)
            complete_sql += chelt_sql_query
            aeroclub_sql_query = self.aeroclub_db.return_sql_text(aeroclub_tables, export_all_users=complete_database)
            complete_sql += aeroclub_sql_query

        output_sql_file = os.path.join(output_dir, '{}_{:09d}.sql'.format(tim, self.id))
        FILE = open(output_sql_file, "w", encoding="utf-8")
        FILE.writelines(complete_sql)
        FILE.close()
        #####
        output_zip = os.path.join(os.path.dirname(output_dir), '{}.zip'.format(output_dir))
        zip_file = self.zip_profile_files(output_dir, output_zip)
        if os.path.exists(zip_file):
            shutil.rmtree(output_dir)
        print('finished backup')
        return output_sql_file

    def backup_users_tables(self, export_files=False):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        user_tables = {}
        for tab in app_tables_users:
            if tab == 'users':
                user_tables[tab] = {'id': self.id}
                continue
            user_tables[tab] = {'id_users': self.id}

        tim = datetime.strftime(datetime.now(), '%Y_%m_%d__%H_%M_%S')
        output_dir = os.path.join(os.path.dirname(__file__), 'static', 'backup_profile',
                                  '{:09d}'.format(self.id),
                                  '{}_{:09d}'.format(tim, self.id))
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        if export_files:
            sql_query = self.users_db.return_sql_text(user_tables, export_files=output_dir)
        else:
            sql_query = self.users_db.return_sql_text(user_tables)

        output_sql_file = os.path.join(output_dir, '{}_{:09d}.sql'.format(tim, self.id))
        FILE = open(output_sql_file, "w", encoding="utf-8")
        FILE.writelines(sql_query)
        FILE.close()
        #####
        output_zip = os.path.join(os.path.dirname(output_dir), '{}.zip'.format(output_dir))
        zip_file = self.zip_profile_files(output_dir, output_zip)
        if os.path.exists(zip_file):
            shutil.rmtree(output_dir)
        print('finished backup')

        return output_sql_file

    def import_zip(self, zip_file, import_files=False):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        output_dir, file = os.path.split(zip_file)
        src_dir = self.unzip_profile_files(zip_file, output_dir)
        src_dir = os.path.join(src_dir, file[:-4])
        if not os.path.exists(src_dir):
            raise RuntimeError('Missing Folder {}'.format(src_dir))

        sql_files = [x for x in os.listdir(src_dir) if x.endswith('.sql')]
        sql_file = os.path.join(src_dir, sql_files[0])
        self.users_db.run_sql_file(sql_file)
        if import_files:
            attachments = [x for x in os.listdir(src_dir) if
                           (x.endswith('.jpg') or
                            x.endswith('.pdf') or
                            x.endswith('.csv') or
                            x.endswith('.CSV')
                            )]
            tab = []
            for file_name in attachments:
                try:
                    # print(file_name)
                    table_id, orig_name = file_name.split('+')
                    fil = os.path.join(src_dir, file_name)
                    self.alimentari.changeCellContent('file', fil, 'id', table_id)
                    self.alimentari.changeCellContent('file_name', str(orig_name), 'id', table_id)
                    # print(user_id, table_name, table_id, orig_name)
                except:
                    print('could not import {}, name not ok'.format(file_name))
                tup = (table_id, orig_name, file_name)
                tab.append(tup)
            # tab = np.atleast_2d(tab)
            # all_sql_tables = list(np.unique(tab[:, 1]))
            # for table_name in all_sql_tables:
            #     # print('table_name', table_name)
            #     sql_table = mysql_rm.Table(self.credentials, table_name)
            #     table = tab[tab[:, 1] == table_name]
            #     for row in table:
            #         user_id, table_name, table_id, orig_name, fl_name = row
            #         # print('&', user_id, table_name, table_id, orig_name, fl_name)
            #         fil = os.path.join(src_dir, fl_name)
            #         sql_table.changeCellContent('file', fil, 'id', table_id)
            #         sql_table.changeCellContent('file_name', str(orig_name), 'id', table_id)
        if os.path.exists(src_dir):
            shutil.rmtree(src_dir)


def main():
    script_start_time = time.time()
    selectedStartDate = datetime(2025, 1, 1, 0, 0, 0)
    selectedEndDate = datetime(2025, 1, 31, 0, 0, 0)

    # if compName == 'DESKTOP-5HHINGF':
    #     ini_file = r"D:\Python\MySQL\cheltuieli.ini"
    # else:
    #     ini_file = r"C:\_Development\Diverse\pypi\cfgm.ini"
    #     # ini_file = r"C:\_Development\Diverse\pypi\cfgmRN.ini"
    users_db_connection = DB_Connection(rappmysql.ini_users)


    user = Users(user_name='emil',
                 users_table=users_db_connection.users_table,
                 user_apps_table=users_db_connection.user_apps_table)
    # zp = r"C:\_Development\Diverse\pypi\rappmysql\src\rappmysql\mruser\static\db_backup\2025_06_26__11_58_11.zip"
    # user.import_zip(zp)
    # user.erase_all_traces()
    print('****', user.admin)
    user.masini = ['aaa', 'bb']
    print(user.masini)
    # print('****', user.masini)
    # print('****', user.cheltuieli)
    # print('****', user.all_possible_applications)
    # print('****', user.unused_applications)
    # print('****', user.applications)
    # user.init_app('cheltuieli')

    # user.init_app('aeroclub')
    # user.backup(complete_database=True)
    # user.export_aeroclub_profile(export_files=True)
    # print('****', user.has_cheltuieli_real)
    # print('****', user.export_profile())
    # print('****', user.export_chelt_plan_sql())
    # print(user.delete_auto('aaaa_aaaa'))
    # if not user.masini:
    #     print('GGG')
    # else:
    #     for aa in user.masini:
    #         print(aa)
    # output_sql = r'C:\_Development\Diverse\pypi\radu\masina\src\masina\static\backup_profile\000000001\hyundai_ioniq.sql'
    # all_cars_ident = {'user_id': 1, 'id': 1}
    # masina_ident = {'id_users': 1, 'id_all_cars': 1}
    # tables = {'all_cars': all_cars_ident,
    #           'masina': masina_ident}

    # dire = r'C:\_Development\Diverse\pypi\radu\masina\src\masina\static\backup_profile\000000001\000000001'
    # sql_query = user.auto_db.return_sql_text(tables, export_files=dire)
    # print(sql_query)
    # user.export_car_sql(2, export_files=True)
    # profile = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\static\backup_profile\000000001\000000002\2025_06_05__16_22_12_000000002.zip"
    # user.import_profile_with_files(profile, import_files=True)

    scrip_end_time = time.time()
    duration = scrip_end_time - script_start_time
    duration = time.strftime("%H:%M:%S", time.gmtime(duration))
    print('run time: {}'.format(duration))


if __name__ == '__main__':
    main()



import sys
import os
import traceback
import decimal
import datetime as dt
from datetime import datetime, timedelta
import time
import numpy as np
from PyQt5.QtWidgets import *
from PyQt5 import uic
from PyQt5 import QtCore
from PyQt5.QtCore import *
# import sip
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import (FigureCanvasQTAgg as FigureCanvas,
                                                NavigationToolbar2QT as NavigationToolbar)
import chelt_plan
from chelt_plan import CheltuieliPlanificate, Income, CheltuieliReale, CheltPlanVSReal, CheltApp
from rappmysql.mruser.myusers import Users
from rappmysql.mruser.myusers import DB_Connection as users_db_connection
from rappmysql.cheltuieli.chelt_plan import DB_Connection as chelt_db_connection
from rappmysql.mysqlquerys import connect
import rappmysql

compName = os.getenv('COMPUTERNAME')


class MyApp(QMainWindow):
    def __init__(self, users_db, chelt_db):
        super(MyApp, self).__init__()

        self.users_db_connection = users_db
        self.chelt_db_connection = chelt_db

        path2src, pyFileName = os.path.split(__file__)
        uiFileName = 'chelt_plan.ui'
        path2GUI = os.path.join(path2src, 'GUI', uiFileName)
        Ui_MainWindow, QtBaseClass = uic.loadUiType(path2GUI)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.conf = connect.Config(rappmysql.ini_chelt)
        self.setWindowTitle('{}'.format(self.conf.credentials['database']))

        self.user = None
        self.app = None
        # self.app_without_logged_user = Users(None, rappmysql.ini_users)

        self.ui.GB_when_loged.setVisible(False)
        self.ui.GB_user_options.setVisible(False)

        # self.prepare_login_window()
        fig = Figure()
        self.add_plots(fig)

        self.ui.PB_login.clicked.connect(self.login_user)
        self.ui.DE_pay_day.setCalendarPopup(True)
        self.ui.PB_export_profile.clicked.connect(self.export_full_profile)
        self.ui.PB_import_profile.clicked.connect(self.import_profile)
        self.ui.PB_DeleteProfile.clicked.connect(self.delete_profile)
        self.ui.PB_fire_req.clicked.connect(self.populate_current_tab_widget)
        self.ui.CBMonths.currentIndexChanged.connect(self.populateDatesInterval)
        self.ui.CB_next_x_days.currentIndexChanged.connect(self.populateDatesInterval)
        self.ui.SB_year.valueChanged.connect(self.populateDatesInterval)
        self.ui.planed_vs_realUnplanned_real_expenses.horizontalHeader().sectionClicked.connect(self.sortUnplanned_real)
        self.ui.planed_vs_realTable.horizontalHeader().sectionClicked.connect(self.sortPlaned_vs_realTable)
        # self.ui.PB_plotTablePie.clicked.connect(self.plotTablePie)
        # self.ui.PB_plotNamePie.clicked.connect(self.plotNamePie)
        # self.ui.PB_Plot.clicked.connect(self.plotGraf)
        self.ui.PB_add_one_time_transactions.clicked.connect(self.add_to_one_time_transactions)
        self.ui.PBComp.clicked.connect(self.compare_real_2plan)
        self.ui.pbImportCSV.clicked.connect(self.importCSV)
        self.ui.PB_backup_planned_expenses.clicked.connect(self.backup_chelt_planned)
        self.ui.TW_main_user_interface.currentChanged.connect(self.populate_current_tab_widget)
        self.ui.CBOrigTableHead.stateChanged.connect(self.populate_current_tab_widget)
        self.ui.CB_exclude_N26.stateChanged.connect(self.populate_current_tab_widget)
        self.ui.CB_hide_planned_but_not_found.stateChanged.connect(self.populate_current_tab_widget)
        self.ui.CB_hide_planned_and_found_in_real_expenses_table.stateChanged.connect(self.populate_current_tab_widget)
        self.ui.CB_hide_unplanned_real_expenses.stateChanged.connect(self.populate_current_tab_widget)
        # self.ui.CB_allHideIntercontotrans.stateChanged.connect(lambda: (self.prepareTabPlanned(), self.prepareTabReal(), self.prepareTabPlanVsReal()))
        self.ui.CB_allHideIntercontotrans.stateChanged.connect(self.populate_current_tab_widget)

        self.ui.planTable.horizontalHeader().sectionClicked.connect(self.sortTable)
        self.ui.realTable.horizontalHeader().sectionClicked.connect(self.sortTable)
        self.ui.TWsskm.horizontalHeader().sectionClicked.connect(self.sortTable)
        self.ui.TWdeubnk.horizontalHeader().sectionClicked.connect(self.sortTable)
        self.ui.TWn26.horizontalHeader().sectionClicked.connect(self.sortTable)
        self.ui.realTable.horizontalHeader().sectionDoubleClicked.connect(self.setFilter)
        self.ui.TWsskm.horizontalHeader().sectionDoubleClicked.connect(self.setFilter)
        self.ui.TWdeubnk.horizontalHeader().sectionDoubleClicked.connect(self.setFilter)
        self.ui.TWn26.horizontalHeader().sectionDoubleClicked.connect(self.setFilter)
        self.ui.RB_show_table.toggled.connect(self.populate_current_tab_widget)
        self.ui.RB_show_tree.toggled.connect(self.populate_current_tab_widget)

        self.ui.planTable.horizontalHeader().setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.ui.planTable.horizontalHeader().customContextMenuRequested.connect(self.contextMenu_plot_cols_from_table)
        self.ui.planed_vs_realUnplanned_real_expenses.horizontalHeader().setContextMenuPolicy(
            QtCore.Qt.CustomContextMenu)
        self.ui.planed_vs_realUnplanned_real_expenses.horizontalHeader().customContextMenuRequested.connect(
            self.contextMenu_plot_cols_from_table)
        self.ui.realTable.horizontalHeader().setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.ui.realTable.horizontalHeader().customContextMenuRequested.connect(self.contextMenu_plot_cols_from_table)
        self.ui.planed_vs_realTable.horizontalHeader().setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.ui.planed_vs_realTable.horizontalHeader().customContextMenuRequested.connect(
            self.contextMenu_plot_cols_from_table)

        if compName == 'DESKTOP-5HHINGF' or compName == 'MPCC6995':
            self.ui.LE_user_name.setText('radu')
            self.ui.LE_user_pass.setText('9876')
            # self.login_user()
            # zipFile = r"C:\_Development\Diverse\pypi\radu\cheltuieli\src\cheltuieli\static\backup_profile\000000001\2024_12_18__14_20_50_000000001.zip"
            # self.app_users.import_profile_with_files(zipFile, import_files=False)
            # self.login_user()

    def delete_profile(self):
        message = "Are you sure you want to delete the profile?"
        buttonReply = QMessageBox.question(self, 'Deletion Prompt', message,
                                           QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Cancel)

        if buttonReply == QMessageBox.Yes:
            self.close()
            # time.sleep(3)
            self.user.chelt_app.erase_chelt_traces()
        else:
            return

    def export_full_profile(self):
        # expName, _ = QFileDialog.getSaveFileName(self, "Save file", "", "File (*.zip)")
        # expDir = QFileDialog.getExistingDirectory()
        # print(expDir)
        # if expName:
        if self.ui.CB_export_profile_with_files.isChecked():
            self.user.chelt_app.export_profile(output_dir=None, export_files=True)
        else:
            self.user.chelt_app.export_profile(output_dir=None, export_files=False)

    def backup_chelt_planned(self):
        self.user.chelt_app.export_chelt_plan_sql()

    def import_profile(self):
        import_files = self.ui.CB_import_profile_with_files.isChecked()
        zip_or_sql_file, _ = QFileDialog.getOpenFileName(self, "Open file", "", "File (*.zip;*.sql)")
        if zip_or_sql_file:
            if zip_or_sql_file.endswith('.zip'):
                zipFile = zip_or_sql_file
                if not self.user:
                    self.app_without_logged_user.import_profile_with_files(zipFile, import_files=import_files)
                    return
                self.user.chelt_app.import_profile(zipFile, import_files=import_files)
                self.login_user()
            elif zip_or_sql_file.endswith('.sql'):
                sqlFile = zip_or_sql_file
                if not self.user:
                    self.app_without_logged_user.import_profile_without_files(sqlFile)
                    return
                self.user.import_profile_without_files(sqlFile)
                self.login_user()

    def login_user(self):
        user_name = self.ui.LE_user_name.text()
        password = self.ui.LE_user_pass.text()

        self.user = Users(user_name=user_name,
                          users_table=self.users_db_connection.users_table,
                          user_apps_table=self.users_db_connection.user_apps_table)

        if self.user.verify_password(password):
            self.ui.GB_when_loged.setVisible(True)
            self.ui.GB_user_options.setVisible(True)
            # self.ui.CB_export_profile_with_files.setVisible(True)
            # self.ui.PB_DeleteProfile.setVisible(True)

            self.user.chelt_app = CheltApp(user_id=self.user.id,
                                           chelt_db=self.chelt_db_connection.chelt_db,
                                           chelt_plan=self.chelt_db_connection.chelt_plan,
                                           yearly_plan=self.chelt_db_connection.yearly_plan,
                                           myAccountsTable=self.chelt_db_connection.myAccountsTable,
                                           income_table=self.chelt_db_connection.income_table)

            if self.user.chelt_app.has_cheltuieli_planned:
                self.app_planned = CheltuieliPlanificate(chelt_db=self.chelt_db_connection.chelt_db,
                                                         chelt_plan=self.chelt_db_connection.chelt_plan,
                                                         yearly_plan=self.chelt_db_connection.yearly_plan)
            else:
                self.ui.GB_when_loged.setVisible(False)
                return
            if self.user.chelt_app.has_planned_income:
                self.app_planned_income = Income(income_table=self.chelt_db_connection.income_table)

            if self.user.chelt_app.has_cheltuieli_real:
                self.app_reale = CheltuieliReale(chelt_db=self.chelt_db_connection.chelt_db,
                                                 myAccountsTable=self.chelt_db_connection.myAccountsTable,
                                                 imported_csv=self.chelt_db_connection.imported_csv,
                                                 chelt_plan=self.chelt_db_connection.chelt_plan,
                                                 plan_vs_real=self.chelt_db_connection.plan_vs_real,
                                                 knowntrans=self.chelt_db_connection.knowntrans,
                                                 sskm=self.chelt_db_connection.sskm,
                                                 deubnk=self.chelt_db_connection.deubnk,
                                                 n26=self.chelt_db_connection.n26)
            else:
                # widget = self.ui.GB_when_loged
                # tabs = widget.findChildren(QTabWidget)
                # print('no_real', tabs)
                # for i, tab in enumerate(tabs):
                #     print(tab)
                #     print(tab.tabText(i))
                tabWidget = self.ui.TW_main_user_interface
                for i in range(tabWidget.count()):
                    if i == 0:
                        continue
                    # print(tabWidget.tabText(i))
                    tabWidget.setTabEnabled(i, False)
            self.populateCBConto()
            self.populateCBMonths()
            self.populateDatesInterval()
            self.populate_current_tab_widget()
        # else:
        #     print('User not registered in database')

    def populate_current_tab_widget(self):
        selectedStartDate = self.ui.DEFrom.date().toPyDate()
        selectedEndDate = self.ui.DEBis.date().toPyDate()
        currentConto = self.ui.cbActiveConto.currentText()
        sql_orig_table_head = self.ui.CBOrigTableHead.isChecked()
        hideintercontotrans = self.ui.CB_allHideIntercontotrans.isChecked()
        self.defaultFilter = True
        self.filterList = []

        chelt_type = self.ui.TW_main_user_interface.tabText(self.ui.TW_main_user_interface.currentIndex())
        if chelt_type == 'Planned Expenses':
            self.app_planned.prepareTablePlan(currentConto, selectedStartDate, selectedEndDate, hideintercontotrans)
            if sql_orig_table_head:
                tableHead = self.app_planned.tableHead
                table = self.app_planned.expenses
            else:
                tableHead = self.app_planned.displayExpensesTableHead
                tableHead, table = chelt_plan.convert_to_display_table(self.app_planned.tableHead,
                                                                       self.app_planned.expenses,
                                                                       self.app_planned.displayExpensesTableHead)
            qt_table_widget = self.ui.planTable
            if self.app_planned.expenses.shape != (1, 0):
                # self.populateExpensesPlan(displayExpensesTableHead)
                if self.ui.RB_show_table.isChecked():
                    self.populateTable(qt_table_widget, tableHead, table)
                    self.ui.TWmnthVSIrreg.setVisible(False)
                    self.ui.planTable.setVisible(True)
                elif self.ui.RB_show_tree.isChecked():
                    self.populateTree()
                    self.ui.planTable.setVisible(False)
                    self.ui.TWmnthVSIrreg.setVisible(True)
                self.populate_expenses_summary()
                self.prep_2Dplot()
            # self.prepareTabPlanned()
        elif chelt_type == 'Planned Income':
            self.app_planned_income.prepareTablePlan(currentConto, selectedStartDate, selectedEndDate)
            if sql_orig_table_head:
                tableHead = self.app_planned_income.tableHead
                table = self.app_planned_income.income
            else:
                tableHead = self.app_planned.displayExpensesTableHead
                tableHead, table = chelt_plan.convert_to_display_table(self.app_planned_income.tableHead,
                                                                       self.app_planned_income.income,
                                                                       self.app_planned_income.displayExpensesTableHead)
            qt_table_widget = self.ui.planTableIncome
            if self.app_planned_income.income.shape != (1, 0):
                self.populateTable(qt_table_widget, tableHead, table)
                self.ui.TWmnthVSIrreg.setVisible(False)
                self.ui.planTable.setVisible(True)
                self.populate_income_summary()
            #     self.prep_2Dplot()
            # self.prepareTabPlanned()
        elif chelt_type == 'Real Expenses':
            qt_table_widget = self.ui.realTable
            self.app_reale.prepareTableReal_new(currentConto, selectedStartDate, selectedEndDate, hideintercontotrans)
            # print('&&&&', self.app_reale.realExpenses.shape)
            if self.app_reale.realExpenses.shape[1] == 0:
                message = "There were no rows found in interval"
                QMessageBox.warning(self, 'Missing Data', message, QMessageBox.Ok)
                return
            if sql_orig_table_head:
                tableHead = self.app_reale.expensesTableReal.columnsNames
                table = self.app_reale.realExpenses
            else:
                tableHead, table = chelt_plan.convert_to_display_table(self.app_reale.plan_vs_real.columnsNames,
                                                                       self.app_reale.realExpenses,
                                                                       self.app_reale.displayRealTableHead)
            self.populateTable(qt_table_widget, tableHead, table)
            self.populate_table_dates()
            # self.populateDatesInterval()
            # self.prepareTabReal()
        elif chelt_type == 'Real Income':
            qt_table_widget = self.ui.realTableIncome
            self.app_reale.prepareTableReal_new(currentConto, selectedStartDate, selectedEndDate, hideintercontotrans)
            if self.app_reale.realIncome.shape[1] == 0:
                message = "There were no rows found in interval"
                QMessageBox.warning(self, 'Missing Data', message, QMessageBox.Ok)
                return
            if sql_orig_table_head:
                tableHead = self.app_reale.plan_vs_real.columnsNames
                table = self.app_reale.realIncome
            else:
                tableHead, table = chelt_plan.convert_to_display_table(self.app_reale.plan_vs_real.columnsNames,
                                                                       self.app_reale.realIncome,
                                                                       self.app_reale.displayRealTableHead)
            self.populateTable(qt_table_widget, tableHead, table)
            self.populate_table_dates()
            # self.populateDatesInterval()
            # self.prepareTabReal()
        elif chelt_type == 'PlanVsReal':
            puffer_days_to_plann = self.ui.SB_puffer_days_to_plann.value()
            self.app_reale.prepareTableReal_new(currentConto, selectedStartDate, selectedEndDate, hideintercontotrans)
            if self.app_reale.realExpenses.shape[1] == 0:
                message = "There were no rows found in interval"
                QMessageBox.warning(self, 'Missing Data', message, QMessageBox.Ok)
                return

            self.planvsReal = CheltPlanVSReal(chelt_db=self.chelt_db_connection.chelt_db,
                                              chelt_plan=self.chelt_db_connection.chelt_plan,
                                              yearly_plan=self.chelt_db_connection.yearly_plan,
                                              myAccountsTable=self.chelt_db_connection.myAccountsTable,
                                              imported_csv=self.chelt_db_connection.imported_csv,
                                              plan_vs_real=self.chelt_db_connection.plan_vs_real,
                                              knowntrans=self.chelt_db_connection.knowntrans,
                                              sskm=self.chelt_db_connection.sskm,
                                              deubnk=self.chelt_db_connection.deubnk,
                                              n26=self.chelt_db_connection.n26,
                                              currentConto=currentConto,
                                              selectedStartDate=selectedStartDate,
                                              selectedEndDate=selectedEndDate,
                                              hideintercontotrans=hideintercontotrans)
            self.planvsReal.find_planned_in_real_expenses_table(hideintercontotrans, puffer_days_to_plann)
            self.planvsReal.find_unplanned_real_expenses(hideintercontotrans, puffer_days_to_plann)
            if self.planvsReal.found_payments_from_planned.shape[0] == 1 and \
                    self.planvsReal.not_found_payments_from_planned.shape[0] == 1 and \
                    self.planvsReal.unplanned_real_expenses.shape[0] == 1:
                return

            min_buchungstag, max_buchungstag = self.app_reale.get_buchungstag_interval(currentConto)
            if min_buchungstag and max_buchungstag:
                self.ui.DE_Buchungstag_min_plvsre.setDate(min_buchungstag)
                self.ui.DE_Buchungstag_max_plvsre.setDate(max_buchungstag)

            if not self.ui.CB_hide_planned_and_found_in_real_expenses_table.isChecked():
                qt_table_planed_vs_realTable = self.ui.planed_vs_realTable
                if sql_orig_table_head:
                    tableHead, table = list(self.planvsReal.found_payments_from_planned[0]), \
                                       self.planvsReal.found_payments_from_planned[1:]
                else:
                    tableHead, table = list(self.planvsReal.found_payments_from_planned_display_table[0]), \
                                       self.planvsReal.found_payments_from_planned_display_table[1:]
                self.populateTable(qt_table_planed_vs_realTable, tableHead, table)
            else:
                self.ui.groupBox_18.setVisible(False)

            if not self.ui.CB_hide_planned_but_not_found.isChecked():
                self.ui.groupBox_19.setVisible(True)
                qt_table_planed_vs_realTable_notFound = self.ui.planed_vs_realTable_notFound
                if sql_orig_table_head:
                    tableHead, table = list(self.planvsReal.not_found_payments_from_planned[0]), \
                                       self.planvsReal.not_found_payments_from_planned[1:]
                else:
                    tableHead, table = list(self.planvsReal.not_found_payments_from_planned_display_table[0]), \
                                       self.planvsReal.not_found_payments_from_planned_display_table[1:]
                self.populateTable(qt_table_planed_vs_realTable_notFound, tableHead, table)
            else:
                self.ui.groupBox_19.setVisible(False)

            if not self.ui.CB_hide_unplanned_real_expenses.isChecked():
                qt_table_planed_vs_realTable_notFound = self.ui.planed_vs_realUnplanned_real_expenses
                if sql_orig_table_head:
                    tableHead, table = list(self.planvsReal.unplanned_real_expenses[0]), \
                                       self.planvsReal.unplanned_real_expenses[1:]
                    if self.ui.CB_exclude_N26.isChecked():
                        tableHead, table = list(self.planvsReal.unplanned_real_expenses_without_N26[0]), \
                                           self.planvsReal.unplanned_real_expenses_without_N26[1:]
                else:
                    tableHead, table = list(self.planvsReal.unplanned_real_expenses[0]), \
                                       self.planvsReal.unplanned_real_expenses[1:]
                    if self.ui.CB_exclude_N26.isChecked():
                        tableHead, table = list(self.planvsReal.unplanned_real_expenses_without_N26[0]), \
                                           self.planvsReal.unplanned_real_expenses_without_N26[1:]
                    tableHead, table = chelt_plan.convert_to_display_table(tableHead,
                                                                           table,
                                                                           self.planvsReal.displayPlanVsRealTableHead)

                self.populateTable(qt_table_planed_vs_realTable_notFound, tableHead, table)
            else:
                self.ui.groupBox_20.setVisible(False)
            self.populate_planvsReal_summary()
        elif chelt_type == 'sskm':
            qt_table_widget = self.ui.TWsskm
            matches = [('Buchungstag', (selectedStartDate, selectedEndDate))]
            table = self.app_reale.sskm.filterRows(matches)
            table = np.atleast_2d(table)
            table_head = self.app_reale.sskm.columnsNames
            if table.shape[1] == 0:
                message = "There were no rows found in interval"
                QMessageBox.warning(self, 'Missing Data', message, QMessageBox.Ok)
                return
            self.populateTable(qt_table_widget, table_head, table)
        elif chelt_type == 'deubnk':
            # self.preparedeubnk()
            qt_table_widget = self.ui.TWdeubnk
            matches = [('Buchungstag', (selectedStartDate, selectedEndDate))]
            table = self.app_reale.deubnk.filterRows(matches)
            table = np.atleast_2d(table)
            table_head = self.app_reale.deubnk.columnsNames
            self.populateTable(qt_table_widget, table_head, table)
        elif chelt_type == 'n26':
            # self.preparen26()
            qt_table_widget = self.ui.TWn26
            matches = [('Buchungstag', (selectedStartDate, selectedEndDate))]
            table = self.app_reale.n26.filterRows(matches)
            table = np.atleast_2d(table)
            # print('ÖÖÖÖ', table.shape)
            table_head = self.app_reale.n26.columnsNames
            self.populateTable(qt_table_widget, table_head, table)

    def populateTable(self, qt_table_widget, tableHead, table):
        qt_table_widget.parent().setVisible(True)
        if table.shape[0] == 0:
            qt_table_widget.parent().setVisible(False)
            return
        qt_table_widget.setColumnCount(len(tableHead))
        qt_table_widget.setHorizontalHeaderLabels(tableHead)
        qt_table_widget.setRowCount(table.shape[0])
        for col in range(table.shape[1]):
            for row in range(table.shape[0]):
                if isinstance(table[row, col], int) or isinstance(table[row, col], float):
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, table[row, col])
                elif isinstance(table[row, col], decimal.Decimal):
                    val = float(table[row, col])
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, val)
                else:
                    item = QTableWidgetItem(str(table[row, col]))
                qt_table_widget.setItem(row, col, item)
        self.ui.LE_table_total_trans.setText(str(table.shape[0]))
        if 'value' in tableHead:
            indxColVal = tableHead.index('value')
        elif 'Betrag' in tableHead:
            indxColVal = tableHead.index('Betrag')
        elif 'Amount' in tableHead:
            indxColVal = tableHead.index('Amount')
        elif 'Debit' in tableHead:
            indxColVal = tableHead.index('Debit')
        elif 'uberweisung' in tableHead:
            indxColVal = tableHead.index('uberweisung')
        # print('***table', type(table), table.shape)
        if table.shape[1] > 0:
            totalValue = sum(table[:, indxColVal].astype(float))
            totalValue = round(totalValue, 2)
            self.ui.LE_table_total_value.setText(str(totalValue))
        header = qt_table_widget.horizontalHeader()
        header.setSectionResizeMode(QHeaderView.ResizeToContents)

    def populate_expenses_summary(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.LEtotalNoOfTransactions.setText(str(self.app_planned.tot_no_of_expenses))
        self.ui.LEtotalValue.setText(str(self.app_planned.tot_val_of_expenses))
        self.ui.LEnoOfMonthly.setText(str(self.app_planned.tot_no_of_monthly_expenses))
        self.ui.LEtotalMonthly.setText(str(self.app_planned.tot_val_of_monthly_expenses))
        self.ui.LEnoOfIrregular.setText(str(self.app_planned.tot_no_of_irregular_expenses))
        self.ui.LEirregular.setText(str(self.app_planned.tot_val_of_irregular_expenses))

    def populate_planvsReal_summary(self):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        if self.ui.CB_exclude_N26.isChecked():
            self.ui.LE_sum_of_unplanned_real_expenses.setText(
                str(self.planvsReal.sum_of_unplanned_real_expenses_without_n26))
        else:
            self.ui.LE_sum_of_unplanned_real_expenses.setText(str(self.planvsReal.sum_of_unplanned_real_expenses))
        self.ui.LE_total_expenses_planned.setText(str(self.planvsReal.sum_planned))
        self.ui.LE_total_expenses_realised.setText(str(self.planvsReal.sum_realised))
        self.ui.LE_total_no_of_transactions_planned.setText(str(self.planvsReal.sum_realised_from_planned_found))
        self.ui.LE_sum_realised_from_planned_found_in_interval.setText(
            str(self.planvsReal.sum_realised_from_planned_found_in_interval))
        self.ui.LE_planned_but_not_realised.setText(
            str(self.planvsReal.sum_realised_from_not_found_payments_from_planned))
        # self.ui.LE_repeated_rows.setText(str(rows_more_than_one_time))
        self.ui.LE_sum_of_unplanned_real_expenses_without_n26.setText(
            str(self.planvsReal.sum_of_unplanned_real_expenses_without_n26))
        self.ui.LE_sum_of_unplanned_real_expenses_only_n26.setText(
            str(self.planvsReal.sum_of_unplanned_real_expenses_only_n26))
        self.ui.LE_sum_of_unplanned_real_expenses_no_category.setText(
            str(self.planvsReal.sum_of_unplanned_real_expenses_no_category))
        self.ui.LE_sum_of_unplanned_real_expenses_with_category.setText(
            str(self.planvsReal.sum_of_unplanned_real_expenses_with_category))

    def importCSV(self):
        print(sys._getframe().f_code.co_name)
        inpFile, _ = QFileDialog.getOpenFileName(None, 'Select .csv file', '', 'CSV files (*.csv)')
        if not inpFile:
            return
        currentConto = self.ui.cbActiveConto.currentText()
        if currentConto == 'all':
            message = 'please select the conto'
            QMessageBox.warning(self, 'Inconsistent Data', message, QMessageBox.Ok)
            return
        # print(inpFile)
        # print(currentConto)
        self.app_reale.import_CSV_new(currentConto, inpFile)

    def delete_items_of_layout(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        layout_grupa = self.gb_available_databases.layout()
        if layout_grupa is not None:
            while layout_grupa.count():
                item = layout_grupa.takeAt(0)
                widget = item.widget()
                if widget is not None:
                    widget.setParent(None)
                # else:
                #     delete_items_of_layout(item.layout())
            # sip.delete(layout_grupa)

    def compare_real_2plan(self):
        print(sys._getframe().f_code.co_name)
        # self.app_reale.compare2plan()
        self.app_reale.find_chelt_plan_rows_in_banks_tables_and_write_to_plan_vs_real_table()
        self.app_reale.find_knowntrans_in_banks_tables_and_write_to_plan_vs_real_table()

    # def totals_PlanVsReal(self, rows_more_than_one_time):
    #     print(sys._getframe().f_code.co_name)
    #     self.ui.LE_total_no_of_transactions_planned.setText(str(self.planvsReal.no_of_transactions_planned))
    #     self.ui.LE_total_expenses_realised.setText(str(self.planvsReal.sum_realised_from_planned_found))
    #
    #     self.ui.LE_no_of_unplanned_real_expenses.setText(str(self.planvsReal.no_of_unplanned_real_expenses))
    #     self.ui.LE_sum_of_unplanned_real_expenses.setText(str(self.planvsReal.sum_of_unplanned_real_expenses))

    def populate_table_dates(self):
        print(sys._getframe().f_code.co_name)
        self.ui.TW_real_table_dates.clear()
        self.ui.TW_real_table_dates.setRowCount(len(self.app_reale.real_table_dates))
        self.ui.TW_real_table_dates.setColumnCount(3)
        self.ui.TW_real_table_dates.setHorizontalHeaderLabels(['Bank', 'From', 'To'])

        for row, (bank, dates) in enumerate(self.app_reale.real_table_dates.items()):
            # print('****',row, bank, dates)
            date_from, date_to = dates[0], dates[1]
            itemBank = QTableWidgetItem(str(bank))
            self.ui.TW_real_table_dates.setItem(row, 0, itemBank)
            itemdate_from = QTableWidgetItem(str(date_from))
            self.ui.TW_real_table_dates.setItem(row, 1, itemdate_from)
            itemdate_to = QTableWidgetItem(str(date_to))
            self.ui.TW_real_table_dates.setItem(row, 2, itemdate_to)

    def populateTableIncome(self, table):
        print(sys._getframe().f_code.co_name)
        self.ui.realTableIncome.setColumnCount(len(self.app_reale.plan_vs_real.columnsNames))
        self.ui.realTableIncome.setHorizontalHeaderLabels(self.app_reale.plan_vs_real.columnsNames)
        self.ui.realTableIncome.setRowCount(table.shape[0])
        for col in range(table.shape[1]):
            for row in range(table.shape[0]):
                if isinstance(table[row, col], int) or isinstance(table[row, col], float):
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, table[row, col])
                elif isinstance(table[row, col], decimal.Decimal):
                    val = float(table[row, col])
                    item = QTableWidgetItem()
                    item.setData(QtCore.Qt.DisplayRole, val)
                else:
                    item = QTableWidgetItem(str(table[row, col]))
                self.ui.realTableIncome.setItem(row, col, item)

        totalVal = 0
        if table.shape[1] > 0:
            allValues = table[:, self.app_reale.plan_vs_real.columnsNames.index('Betrag')].astype(float)
            if None in allValues:
                allValues = allValues[allValues != np.array(None)]
            totalVal = sum(allValues)
        self.ui.LEtotalNoIncomeTrans.setText(str(len(table)))
        self.ui.LEtotalIncomeValue.setText(str(totalVal))

    def totals_r(self):
        print(sys._getframe().f_code.co_name)
        if self.ui.LEtotalNoExpensesTrans.text():
            expensesTrans = int(self.ui.LEtotalNoExpensesTrans.text())
        else:
            expensesTrans = 0
        if self.ui.LEtotalNoIncomeTrans.text():
            incomeTrans = int(self.ui.LEtotalNoIncomeTrans.text())
        else:
            incomeTrans = 0

        if self.ui.LEtotalExpensesValue.text():
            expenses = float(self.ui.LEtotalExpensesValue.text())
        else:
            expenses = 0
        if self.ui.LEtotalIncomeValue.text():
            income = float(self.ui.LEtotalIncomeValue.text())
        else:
            income = 0

        trans = expensesTrans + incomeTrans
        total = expenses + income
        print('trans', trans)
        print('total', total)
        # self.ui.LEtotalNoOfRealTransactions.setText(str(trans))
        # self.ui.LEtotalRealValue.setText(str(total))

    def populateCBMonths(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.CBMonths.addItem('month')
        months = [dt.date(2000, m, 1).strftime('%B') for m in range(1, 13)]
        for month in months:
            self.ui.CBMonths.addItem(month)

        self.ui.SB_year.setValue(datetime.now().year)

    def populateCBConto(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.cbActiveConto.addItem('all')
        self.ui.cbActiveConto.addItems(self.app_planned.myContos)
        self.ui.cballContos.addItems(self.app_planned.myContos)

    def populateDatesInterval(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        chelt_type = self.ui.TW_main_user_interface.tabText(self.ui.TW_main_user_interface.currentIndex())
        if chelt_type == "Planned Expenses":
            # startDate, lastDayOfMonth = chelt_plan.default_interval()
            startDate, lastDayOfMonth = self.app_planned.default_interval()
        else:
            startDate, lastDayOfMonth = self.app_reale.default_interval()

        if self.ui.CBMonths.currentText() != 'month':
            year = int(self.ui.SB_year.value())
            startDate, lastDayOfMonth = chelt_plan.get_monthly_interval(self.ui.CBMonths.currentText(), year)

        if self.ui.CB_next_x_days.currentText() != '':
            # print(self.ui.CB_next_x_days.currentText())
            txt = self.ui.CB_next_x_days.currentText()
            days = txt.strip(' days')
            days = int(days)
            lastDayOfMonth = chelt_plan.calculate_today_plus_x_days(days)

        self.ui.DEFrom.setDate(startDate)
        self.ui.DEBis.setDate(lastDayOfMonth)

        self.ui.DEFrom.setCalendarPopup(True)
        self.ui.DEBis.setCalendarPopup(True)

    def populateTree(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.TWmnthVSIrreg.clear()
        if self.ui.CBOrigTableHead.isChecked():
            tableHead = self.app_planned.tableHead.copy()
            tableHead.insert(0, 'freq')
            self.ui.TWmnthVSIrreg.setHeaderLabels(tableHead)
        else:
            self.ui.TWmnthVSIrreg.setHeaderLabels(['freq', 'name', 'value'])
        monthly_level = QTreeWidgetItem(self.ui.TWmnthVSIrreg)
        monthly_level.setText(0, 'Monthly')
        irregular_level = QTreeWidgetItem(self.ui.TWmnthVSIrreg)
        irregular_level.setText(0, 'Irregular')
        monthlyIndx = np.where(self.app_planned.expenses[:, self.app_planned.tableHead.index('freq')] == 1)
        monthly = self.app_planned.expenses[monthlyIndx]
        for mnth in monthly:
            mnth_item_level = QTreeWidgetItem(monthly_level)
            if self.ui.CBOrigTableHead.isChecked():
                for iii, col in enumerate(mnth):
                    mnth_item_level.setText(iii + 1, str(col))
            else:
                mnth_item_level.setText(1, mnth[self.app_planned.tableHead.index('name')])
                mnth_item_level.setText(2, str(round(mnth[self.app_planned.tableHead.index('value')])))

        totalMonthly = self.app_planned.expenses[monthlyIndx, self.app_planned.tableHead.index('value')][0]
        monthly_level.setText(1, 'Total')
        monthly_level.setText(2, str(round(sum(totalMonthly), 2)))

        irregIndx = np.where(self.app_planned.expenses[:, self.app_planned.tableHead.index('freq')] != 1)
        irregular = self.app_planned.expenses[irregIndx]
        for irr in irregular:
            irr_item_level = QTreeWidgetItem(irregular_level)
            if self.ui.CBOrigTableHead.isChecked():
                for iii, col in enumerate(irr):
                    irr_item_level.setText(iii + 1, str(col))
            else:
                irr_item_level.setText(1, irr[self.app_planned.tableHead.index('name')])
                irr_item_level.setText(2, str(round(irr[self.app_planned.tableHead.index('value')], 2)))

        totalIrreg = self.app_planned.expenses[irregIndx, self.app_planned.tableHead.index('value')][0]
        irregular_level.setText(1, 'Total')
        irregular_level.setText(2, str(round(sum(totalIrreg), 2)))

    def populate_income_summary(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.ui.LE_total_netto.setText(str(self.app_planned_income.netto))
        self.ui.LE_total_taxes.setText(str(self.app_planned_income.taxes))
        self.ui.LE_total_brutto.setText(str(self.app_planned_income.brutto))
        self.ui.LE_salary_uberweisung.setText(str(self.app_planned_income.salary_uberweisung))
        self.ui.LE_salary_abzuge.setText(str(self.app_planned_income.salary_abzuge))
        self.ui.LE_salary_netto.setText(str(self.app_planned_income.salary_netto))
        self.ui.LE_salary_gesetzliche_abzuge.setText(str(self.app_planned_income.salary_gesetzliche_abzuge))
        self.ui.LE_salary_brutto.setText(str(self.app_planned_income.salary_brutto))

    def sortUnplanned_real(self, logical_index):
        print(sys._getframe().f_code.co_name)
        header = self.ui.planed_vs_realUnplanned_real_expenses.horizontalHeader()
        order = Qt.DescendingOrder
        if not header.isSortIndicatorShown():
            header.setSortIndicatorShown(True)
        elif header.sortIndicatorSection() == logical_index:
            order = header.sortIndicatorOrder()
        header.setSortIndicator(logical_index, order)
        self.ui.planed_vs_realUnplanned_real_expenses.sortItems(logical_index, order)

    def sortPlaned_vs_realTable(self, logical_index):
        print(sys._getframe().f_code.co_name)
        header = self.ui.planed_vs_realTable.horizontalHeader()
        order = Qt.DescendingOrder
        if not header.isSortIndicatorShown():
            header.setSortIndicatorShown(True)
        elif header.sortIndicatorSection() == logical_index:
            order = header.sortIndicatorOrder()
        header.setSortIndicator(logical_index, order)
        self.ui.planed_vs_realTable.sortItems(logical_index, order)

    # def readPlanExpenses(self):
    #     # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     rows = self.ui.planTable.rowCount()
    #     cols = self.ui.planTable.columnCount()
    #     planExpenseTable = np.empty((rows, cols), dtype=object)
    #     planExpenseTableHead = []
    #     for row in range(rows):
    #         for column in range(cols):
    #             cell = self.ui.planTable.item(row, column)
    #             planExpenseTable[row, column] = cell.text()
    #             colName = self.ui.planTable.horizontalHeaderItem(column).text()
    #             if colName not in planExpenseTableHead:
    #                 planExpenseTableHead.append(colName)
    #
    #     return planExpenseTable, planExpenseTableHead
    #
    # def readPlanIncome(self):
    #     # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     rows = self.ui.planTableIncome.rowCount()
    #     cols = self.ui.planTableIncome.columnCount()
    #     planIncomeTable = np.empty((rows, cols), dtype=object)
    #     planIncomeTableHead = []
    #     for row in range(rows):
    #         for column in range(cols):
    #             cell = self.ui.planTableIncome.item(row, column)
    #             planIncomeTable[row, column] = cell.text()
    #             colName = self.ui.planTableIncome.horizontalHeaderItem(column).text()
    #             print('000colName', colName)
    #             if colName not in planIncomeTableHead:
    #                 planIncomeTableHead.append(colName)
    #     print('****planIncomeTableHead', planIncomeTableHead)
    #     return planIncomeTable, planIncomeTableHead

    # def plotGraf(self):
    #     # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     realExpenseTable, realExpenseTableHead = self.readPlanExpenses()
    #     planIncomeTable, planIncomeTableHead = self.readPlanIncome()
    #     print('****planIncomeTableHead', planIncomeTableHead)
    #     x_exp = []
    #     y_exp = []
    #     for date in np.unique(realExpenseTable[:, realExpenseTableHead.index('payDay')]):
    #         indx = np.where(realExpenseTable[:, realExpenseTableHead.index('payDay')] == date)
    #         arr = realExpenseTable[indx, realExpenseTableHead.index('value')].astype(float)
    #         x_exp.append(date)
    #         y_exp.append(abs(sum(arr[0])))
    #
    #     x_inc = []
    #     y_inc = []
    #     for date in np.unique(planIncomeTable[:, planIncomeTableHead.index('payDay')]):
    #         indx = np.where(planIncomeTable[:, planIncomeTableHead.index('payDay')] == date)
    #         arr = planIncomeTable[indx, planIncomeTableHead.index('value')].astype(float)
    #         x_inc.append(date)
    #         y_inc.append(abs(sum(arr[0])))
    #
    #     fig1, ax1 = plt.subplots()
    #     ax1.plot(x_exp, y_exp)
    #     ax1.plot(x_inc, y_inc)
    #     # plt.setp(plt.get_xticklabels(), rotation=30, ha="right")
    #     fig1.autofmt_xdate()
    #     plt.grid()
    #     plt.show()

    def add_to_one_time_transactions(self):
        name = self.ui.LE_add_name.text()
        value = float(self.ui.LE_add_value.text())
        myconto = self.ui.cballContos.currentText()

        pay_day = self.ui.DE_pay_day.date().toPyDate()
        self.app_planned.add_one_time_transactions(name, value, myconto, pay_day)

    def sortTable(self, logical_index):
        print(sys._getframe().f_code.co_name)

        chelt_type = self.ui.TW_main_user_interface.tabText(self.ui.TW_main_user_interface.currentIndex())
        if chelt_type == 'Real Expenses':
            table = self.ui.realTable
        elif chelt_type == 'Planned Expenses':
            table = self.ui.planTable
        elif chelt_type == 'sskm':
            table = self.ui.TWsskm
        elif chelt_type == 'deubnk':
            table = self.ui.TWdeubnk
        elif chelt_type == 'n26':
            table = self.ui.TWn26

        header = table.horizontalHeader()
        order = Qt.DescendingOrder
        if not header.isSortIndicatorShown():
            header.setSortIndicatorShown(True)
        elif header.sortIndicatorSection() == logical_index:
            order = header.sortIndicatorOrder()
        header.setSortIndicator(logical_index, order)
        table.sortItems(logical_index, order)

    def setFilter(self, logical_index):
        print(sys._getframe().f_code.co_name)

        chelt_type = self.ui.TW_main_user_interface.tabText(self.ui.TW_main_user_interface.currentIndex())
        if chelt_type == 'Real Expenses':
            qt_table = self.ui.realTable
            sql_table = self.app_reale.plan_vs_real
        elif chelt_type == 'Planned Expenses':
            qt_table = self.ui.planTable
            sql_table = self.app_reale.chelt_plan
        elif chelt_type == 'sskm':
            qt_table = self.ui.TWsskm
            sql_table = self.app_reale.sskm
        elif chelt_type == 'deubnk':
            qt_table = self.ui.TWdeubnk
            sql_table = self.app_reale.deubnk
        elif chelt_type == 'n26':
            qt_table = self.ui.TWn26
            sql_table = self.app_reale.n26
        # print('ÖÖÖÖÖÖÖ', qt_table.objectName())
        # print('logical_index', logical_index)
        colName = qt_table.horizontalHeaderItem(logical_index).text()
        # print('colName', colName)
        # return
        # colName = self.plan_vs_real.columnsNames[logical_index]
        colType = sql_table.get_column_type(colName)
        if colType == 'int':
            filt = FilterWindow.getIntInterval(colName)
            if not filt:
                return
            if isinstance(filt, tuple):
                minInt, maxInt = filt
                filterVals = (str(minInt), str(maxInt))
            elif isinstance(filt, str):
                filterVals = filt
            self.applyFilter(colName, filterVals)
        elif colType == 'date':
            filt = FilterWindow.getDateInterval(colName)
            if not filt:
                return
            if isinstance(filt, tuple):
                minDate, maxDate = filt
                minDate = minDate.toPyDate()
                maxDate = maxDate.toPyDate()
                filterVals = (str(minDate), str(maxDate))
            elif isinstance(filt, str):
                filterVals = filt
            self.applyFilter(colName, filterVals)
        else:
            header = qt_table.horizontalHeader()
            geom = QtCore.QRect(header.sectionViewportPosition(logical_index), 0, header.sectionSize(logical_index),
                                header.height())
            item = QLineEdit(header)
            item.setGeometry(geom)
            item.show()
            item.setFocus()
            item.editingFinished.connect(lambda: (self.applyFilter(colName, item.text(), qt_table, sql_table),
                                                  item.clear(),
                                                  item.hide(),
                                                  item.deleteLater()))

    def applyFilter(self, colName, filter, qt_table, sql_table):
        print(sys._getframe().f_code.co_name)
        if filter == '':
            return

        if self.defaultFilter:
            self.ui.lineEditFilterList.clear()
            self.defaultFilter = False

        filterText = self.ui.lineEditFilterList.text()
        if not filterText:
            if isinstance(filter, str):
                filterText += '{}="{}"'.format(colName, filter)
            elif isinstance(filter, tuple):
                filterText += '{} < {} < {}'.format(filter[0], colName, filter[1])
            elif isinstance(filter, list):
                filterText += '{} in {}"'.format(str(filter), colName)
        else:
            if isinstance(filter, str):
                filterText += '; {}="{}"'.format(colName, filter)
            elif isinstance(filter, tuple):
                filterText += '; {} < {} < {}'.format(filter[0], colName, filter[1])
            elif isinstance(filter, list):
                filterText += '; {} in {}"'.format(str(filter), colName)
        self.ui.lineEditFilterList.setText(filterText)
        tup = (colName, filter)
        self.filterList.append(tup)
        selectedStartDate = self.ui.DEFrom.date().toPyDate()
        selectedEndDate = self.ui.DEBis.date().toPyDate()
        match2 = ('Buchungstag', (selectedStartDate, selectedEndDate))
        self.filterList.append(match2)

        currentConto = self.ui.cbActiveConto.currentText()
        if currentConto != 'all':
            match3 = ('myconto', currentConto)
            self.filterList.append(match3)

        filtered_vals = sql_table.filterRows(self.filterList)
        filtered_vals = np.atleast_2d(filtered_vals)
        if filtered_vals.shape == (1, 0):
            filtered_vals = np.empty((0, len(sql_table.columnsNames)))

        sql_orig_table_head = self.ui.CBOrigTableHead.isChecked()
        if qt_table.objectName() == 'realTable':
            # self.populateTableReal(filtered_vals)
            if sql_orig_table_head:
                tableHead = self.app_reale.plan_vs_real.columnsNames
                table = self.app_reale.realExpenses
            else:
                tableHead, table = chelt_plan.convert_to_display_table(self.app_reale.plan_vs_real.columnsNames,
                                                                       filtered_vals,
                                                                       self.app_reale.displayRealTableHead)
            self.populateTable(qt_table, tableHead, table)
        if qt_table.objectName() == 'TWsskm' or qt_table.objectName() == 'TWdeubnk' or qt_table.objectName() == 'TWn26':
            # self.preparesskm(filtered_vals)
            self.populateTable(qt_table, sql_table.columnsNames, filtered_vals)

    # def plotTablePie(self):
    #     # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     realExpenseTable, realExpenseTableHead = self.readPlanExpenses()
    #     allValues = realExpenseTable[:, realExpenseTableHead.index('value')].astype(float)
    #     if None in allValues:
    #         allValues = allValues[allValues != np.array(None)]
    #     totalVal = sum(allValues)
    #
    #     colTableName = realExpenseTable[:, realExpenseTableHead.index('category')]
    #     labels = []
    #     sizes = []
    #     for table in np.unique(colTableName):
    #         indx = np.where(realExpenseTable[:, realExpenseTableHead.index('category')] == table)
    #         smallArray = realExpenseTable[indx]
    #         values = sum(smallArray[:, realExpenseTableHead.index('value')].astype(float))
    #         txt = '{} = {:.2f}'.format(table, values)
    #         labels.append(txt)
    #         size = (values / totalVal) * 100
    #         sizes.append(size)
    #
    #     fig1, ax1 = plt.subplots()
    #     ax1.pie(sizes, labels=labels, autopct='%1.2f%%', startangle=90)
    #     ax1.axis('equal')
    #     plt.legend(title='Total: {:.2f}'.format(totalVal))
    #
    #     plt.show()

    # def plotNamePie(self):
    #     # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     realExpenseTable, realExpenseTableHead = self.readPlanExpenses()
    #     allValues = realExpenseTable[:, realExpenseTableHead.index('value')].astype(float)
    #     if None in allValues:
    #         allValues = allValues[allValues != np.array(None)]
    #     totalVal = sum(allValues)
    #
    #     colTableName = realExpenseTable[:, realExpenseTableHead.index('name')]
    #     labels = []
    #     sizes = []
    #     for table in np.unique(colTableName):
    #         indx = np.where(realExpenseTable[:, realExpenseTableHead.index('name')] == table)
    #         smallArray = realExpenseTable[indx]
    #         values = sum(smallArray[:, realExpenseTableHead.index('value')].astype(float))
    #         txt = '{} = {:.2f}'.format(table, values)
    #         labels.append(txt)
    #         size = (values / totalVal) * 100
    #         sizes.append(size)
    #
    #     fig1, ax1 = plt.subplots()
    #     ax1.pie(sizes, labels=labels, autopct='%1.2f%%', startangle=90)
    #     ax1.axis('equal')
    #     plt.legend(title='Total: {:.2f}'.format(totalVal))
    #
    #     plt.show()

    # def plotPie(self, cols):
    #     print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
    #     realExpenseTable, realExpenseTableHead = self.readPlanExpenses()
    #     try:
    #         allValues = realExpenseTable[:, realExpenseTableHead.index(cols[0])].astype(float)
    #         if None in allValues:
    #             allValues = allValues[allValues != np.array(None)]
    #         totalVal = sum(allValues)
    #         values_col = cols[0]
    #         names_col = cols[1]
    #     except:
    #         allValues = realExpenseTable[:, realExpenseTableHead.index(cols[1])].astype(float)
    #         if None in allValues:
    #             allValues = allValues[allValues != np.array(None)]
    #         totalVal = sum(allValues)
    #         values_col = cols[1]
    #         names_col = cols[0]
    #
    #     colTableName = realExpenseTable[:, realExpenseTableHead.index(names_col)]
    #     labels = []
    #     sizes = []
    #     for table in np.unique(colTableName):
    #         indx = np.where(realExpenseTable[:, realExpenseTableHead.index(names_col)] == table)
    #         smallArray = realExpenseTable[indx]
    #         values = sum(smallArray[:, realExpenseTableHead.index(values_col)].astype(float))
    #         txt = '{} = {:.2f}'.format(table, values)
    #         labels.append(txt)
    #         size = (values / totalVal) * 100
    #         sizes.append(size)
    #
    #     fig1, ax1 = plt.subplots()
    #     ax1.pie(sizes, labels=labels, autopct='%1.2f%%', startangle=90)
    #     ax1.axis('equal')
    #     plt.legend(title='Total: {:.2f}'.format(totalVal))
    #
    #     plt.show()

    def prep_2Dplot(self):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        # realExpenseTable, realExpenseTableHead = self.readPlanExpenses()
        qtablewidget = self.ui.planTable
        realExpenseTable, realExpenseTableHead = self.read_table(qtablewidget)
        allValues = realExpenseTable[:, realExpenseTableHead.index('value')].astype(float)
        if None in allValues:
            allValues = allValues[allValues != np.array(None)]
        totalVal = sum(allValues)

        colTableName = realExpenseTable[:, realExpenseTableHead.index('category')]
        labels = []
        sizes = []
        for table in np.unique(colTableName):
            indx = np.where(realExpenseTable[:, realExpenseTableHead.index('category')] == table)
            smallArray = realExpenseTable[indx]
            values = sum(smallArray[:, realExpenseTableHead.index('value')].astype(float))
            txt = '{} = {:.2f}'.format(table, values)
            labels.append(txt)
            size = (values / totalVal) * 100
            sizes.append(size)

        fig1 = Figure()
        spec5 = fig1.add_gridspec(ncols=1, nrows=2, height_ratios=[2, 1])
        ax1f1 = fig1.add_subplot(spec5[0, 0])  #
        ax1f1.grid(True)
        ax1f1.pie(sizes, labels=labels, autopct='%1.2f%%', startangle=90)
        ax1f1.axis('equal')
        ax1f1.legend()

        ax1f2 = fig1.add_subplot(spec5[1, 0])
        ax1f2.grid(True)

        payments_dict, labels = self.app_planned.prep_yearly_graf()
        ##########################################################################
        # monthly_plus_irregular = payments_dict['monthly+irregular']
        # monthly = payments_dict['monthly']
        # irregular = payments_dict['irregular']
        x = np.arange(len(labels))  # the label locations
        # width = 0.35  # the width of the bars
        # rects1 = ax1f2.bar(x - width / 2, monthly_plus_irregular, width, label='monthly+irregular')
        # rects2 = ax1f2.bar(x + width / 2, monthly, width, label='monthly')
        ##########################################################################
        sex_counts = {
            'monthly': payments_dict['monthly'],
            'irregular': payments_dict['irregular'],
        }
        bottom = np.zeros(len(labels))
        width = 0.6  # the width of the bars: can also be len(x) sequence
        for sex, sex_count in sex_counts.items():
            # print(bottom)
            p = ax1f2.bar(labels, sex_count, width, label=sex, bottom=bottom)
            bottom += sex_count

            ax1f2.bar_label(p, label_type='center')
        ##########################################################################

        ax1f2.set_xticks(x)
        ax1f2.set_xticklabels(labels)
        ax1f2.tick_params(axis='x', rotation=55)
        ax1f2.legend()
        self.rmmpl()
        self.add_plots(fig1)

    def add_plots(self, fig):
        # print(sys._getframe().f_code.co_name)
        self.canvas = FigureCanvas(fig)
        self.ui.loc_de_plot.addWidget(self.canvas)

    def rmmpl(self):
        # print(sys._getframe().f_code.co_name)
        self.ui.loc_de_plot.removeWidget(self.canvas)
        self.canvas.close()

    # def contextTableList(self, event):
    #     # print(sys._getframe().f_code.co_name)
    #     contextMenu = QMenu(self)
    #     item = self.ui.planTable.itemAt(event)
    #     if item is not None:
    #         table2Delete = item.text()
    #         contextMenu.addAction("plot pie")
    #         # contextMenu.addAction("delete all data in table")
    #         # contextMenu.addAction("delete")
    #     action = contextMenu.exec_(self.ui.planTable.mapToGlobal(event))
    #
    #     if action is not None:
    #         if action.text() == "plot pie":
    #             selectedColsIndexes = self.ui.planTable.selectionModel().selectedColumns()
    #             cols = []
    #             for tt in selectedColsIndexes:
    #                 # print(tt, type(tt))
    #                 # print(tt.row(), tt.column())
    #                 col_name = self.ui.planTable.horizontalHeaderItem(tt.column()).text()
    #                 cols.append(col_name)
    #             self.plotPie(cols)

    def contextMenu_plot_cols_from_table(self, event):
        # print(sys._getframe().f_code.co_name)
        # print('self.sender()', self.sender().parent())
        # print('self.sender()', self.sender().parent().objectName())
        qtablewidget = self.sender().parent()
        contextMenu = QMenu(self)
        item = qtablewidget.itemAt(event)
        if item is not None:
            contextMenu.addAction("plot pie")
        action = contextMenu.exec_(qtablewidget.mapToGlobal(event))

        if action is not None:
            if action.text() == "plot pie":
                selectedColsIndexes = qtablewidget.selectionModel().selectedColumns()
                cols = []
                for tt in selectedColsIndexes:
                    # print(tt, type(tt))
                    # print(tt.row(), tt.column())
                    col_name = qtablewidget.horizontalHeaderItem(tt.column()).text()
                    cols.append(col_name)
                print(cols)
                self.plotPie_new(cols, qtablewidget)

    def read_table(self, qtablewidget):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        rows = qtablewidget.rowCount()
        cols = qtablewidget.columnCount()
        planExpenseTable = np.empty((rows, cols), dtype=object)
        planExpenseTableHead = []
        for row in range(rows):
            for column in range(cols):
                cell = qtablewidget.item(row, column)
                planExpenseTable[row, column] = cell.text()
                colName = qtablewidget.horizontalHeaderItem(column).text()
                if colName not in planExpenseTableHead:
                    planExpenseTableHead.append(colName)

        return planExpenseTable, planExpenseTableHead

    def plotPie_new(self, cols, qtablewidget):
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        realExpenseTable, realExpenseTableHead = self.read_table(qtablewidget)
        # return
        try:
            allValues = realExpenseTable[:, realExpenseTableHead.index(cols[0])].astype(float)
            if None in allValues:
                allValues = allValues[allValues != np.array(None)]
            totalVal = sum(allValues)
            values_col = cols[0]
            names_col = cols[1]
        except:
            allValues = realExpenseTable[:, realExpenseTableHead.index(cols[1])].astype(float)
            if None in allValues:
                allValues = allValues[allValues != np.array(None)]
            totalVal = sum(allValues)
            values_col = cols[1]
            names_col = cols[0]

        colTableName = realExpenseTable[:, realExpenseTableHead.index(names_col)]
        labels = []
        sizes = []
        for table in np.unique(colTableName):
            indx = np.where(realExpenseTable[:, realExpenseTableHead.index(names_col)] == table)
            smallArray = realExpenseTable[indx]
            values = sum(smallArray[:, realExpenseTableHead.index(values_col)].astype(float))
            txt = '{} = {:.2f}'.format(table, values)
            labels.append(txt)
            size = (values / totalVal) * 100
            sizes.append(size)

        fig1, ax1 = plt.subplots()
        ax1.pie(sizes, labels=labels, autopct='%1.2f%%', startangle=90)
        ax1.axis('equal')
        plt.legend(title='Total: {:.2f}'.format(totalVal))

        plt.show()

    # def applyFilter_old(self, colName, filter):
    #     print(sys._getframe().f_code.co_name)
    #     if filter == '':
    #         return
    #
    #     if self.defaultFilter:
    #         self.ui.lineEditFilterList.clear()
    #         self.defaultFilter = False
    #
    #     filterText = self.ui.lineEditFilterList.text()
    #     if not filterText:
    #         if isinstance(filter, str):
    #             filterText += '{}="{}"'.format(colName, filter)
    #         elif isinstance(filter, tuple):
    #             filterText += '{} < {} < {}'.format(filter[0], colName, filter[1])
    #         elif isinstance(filter, list):
    #             filterText += '{} in {}"'.format(str(filter), colName)
    #     else:
    #         if isinstance(filter, str):
    #             filterText += '; {}="{}"'.format(colName, filter)
    #         elif isinstance(filter, tuple):
    #             filterText += '; {} < {} < {}'.format(filter[0], colName, filter[1])
    #         elif isinstance(filter, list):
    #             filterText += '; {} in {}"'.format(str(filter), colName)
    #     self.ui.lineEditFilterList.setText(filterText)
    #     tup = (colName, filter)
    #     self.filterList.append(tup)
    #     selectedStartDate = self.ui.DEFrom.date().toPyDate()
    #     selectedEndDate = self.ui.DEBis.date().toPyDate()
    #     match2 = ('Buchungstag', (selectedStartDate, selectedEndDate))
    #     self.filterList.append(match2)
    #
    #     currentConto = self.ui.cbActiveConto.currentText()
    #     if currentConto != 'all':
    #         match3 = ('myconto', currentConto)
    #         self.filterList.append(match3)
    #
    #     realExpenses = self.app_reale.plan_vs_real.filterRows(self.filterList)
    #     # print(50*'Ü')
    #     # for i in self.filterList:
    #     #     print('--', i)
    #     # print()
    #     payments, income = self.app_reale.split_expenses_income(realExpenses)
    #     realExpenses = np.atleast_2d(payments)
    #     realIncome = np.atleast_2d(income)
    #     if realExpenses.shape == (1, 0):
    #         realExpenses = np.empty((0, len(self.app_reale.plan_vs_real.columnsNames)))
    #     if realIncome.shape == (1, 0):
    #         realIncome = np.empty((0, len(self.app_reale.plan_vs_real.columnsNames)))
    #     self.populateTableReal(realExpenses)
    #     self.populateTableIncome(realIncome)


class FilterWindow(QDialog):
    def __init__(self, colType, colName):
        super(FilterWindow, self).__init__()
        path2src, pyFileName = os.path.split(__file__)
        uiFileName = 'filterWindow.ui'
        path2GUI = os.path.join(path2src, 'GUI', uiFileName)
        Ui_MainWindow, QtBaseClass = uic.loadUiType(path2GUI)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.colName = colName
        self.ui.GB_DateInterval.setVisible(False)
        self.ui.GB_IntInterval.setVisible(False)
        self.ui.checkBoxDateInterval.stateChanged.connect(self.openDateInterval)
        self.ui.checkBoxIntInterval.stateChanged.connect(self.openIntInterval)

        self.rejected.connect(self.byebye)

        if colType == 'int':
            self.ui.GB_IntInterval.setVisible(True)
            self.ui.lineEdit_max.setEnabled(False)
        if colType == 'date':
            self.ui.GB_DateInterval.setVisible(True)
            self.ui.dateEditTo.setEnabled(False)

    def openIntInterval(self):
        if self.ui.checkBoxIntInterval.isChecked():
            self.ui.lineEdit_max.setEnabled(True)
            self.ui.label_int.setText('< {} <'.format(self.colName))
        else:
            self.ui.lineEdit_max.setEnabled(False)
            self.ui.label_int.setText('= {}'.format(self.colName))

    def openDateInterval(self):
        if self.ui.checkBoxDateInterval.isChecked():
            self.ui.dateEditTo.setEnabled(True)
            self.ui.label_date.setText('< {} <'.format(self.colName))
        else:
            self.ui.dateEditTo.setEnabled(False)
            self.ui.label_date.setText('= {}'.format(self.colName))

    def byebye(self):
        self.close()

    def intInterval(self):
        if self.ui.checkBoxIntInterval.isChecked():
            tup = (self.ui.lineEdit_min.text(), self.ui.lineEdit_max.text())
            return tup
        else:
            return self.ui.lineEdit_min.text()

    def dateInterval(self):
        if self.ui.checkBoxDateInterval.isChecked():
            tup = (self.ui.dateEditFrom.date(), self.ui.dateEditTo.date())
            return tup
        else:
            return self.ui.dateEditFrom.date()

    @staticmethod
    def getIntInterval(colName):
        dialog = FilterWindow('int', colName)
        result = dialog.exec_()
        filt = dialog.intInterval()
        if result == QDialog.Accepted:
            return filt
        else:
            return None

    @staticmethod
    def getDateInterval(colName):
        dialog = FilterWindow('date', colName)
        result = dialog.exec_()
        filt = dialog.dateInterval()
        if result == QDialog.Accepted:
            return filt
        else:
            return None


def main():
    chelt_db = chelt_db_connection(rappmysql.ini_chelt)
    users_db = users_db_connection(rappmysql.ini_users)

    app = QApplication(sys.argv)
    window = MyApp(users_db, chelt_db)
    window.show()
    # sys.exit(app.exec_())
    app.exec_()


if __name__ == '__main__':
    main()
